/*
 *--------------------------------------------------------------------
 *
 * Font
 *
 * Funkce pro praci s fonty a texty jimi vykreslovane
 *
 * verze   1.0.1 - 14.4.2000
 * autori  Vojvo
 *
 *-------------------------------------------------------------------------
 * zmeny
 *
 * 1.0.1  - snaha o prvni plnou verzi
 * 0.10.1 - *putString* pridava k pripadne vyjimce sve info
 * 0.9.4 - getPutString vraci rect
 * 0.8.1 - putString dava do vyjimky i text, ktery ji zpusobil
 * 0.0.2 - Font predelan na strukturu, nese sebou informace o svych
 *         vlastnostech
 *       - pridany funkce putStringCentered() a getPutString()
 * 0.0.1 - prvni verze
 *-------------------------------------------------------------------------
 */

#include <stdio.h>
#include "utils.h"
#include "graphDD.h"
#include "surfaceManager.h"
#include "exceptions.h"
#include "font.h"

#define QFONT_XAND      15
#define QFONT_YSHIFT    4
#define SFONT_XAND      31
#define SFONT_YSHIFT    5

Font qfont_r = { NULL, 0, 0, QFONT_XAND, QFONT_YSHIFT, 16, 32 };
Font qfont_g = { NULL, 0, 0, QFONT_XAND, QFONT_YSHIFT, 16, 32 };
Font qfont_b = { NULL, 0, 0, QFONT_XAND, QFONT_YSHIFT, 16, 32 };
Font qfont_y = { NULL, 0, 0, QFONT_XAND, QFONT_YSHIFT, 16, 32 };
Font qfont_w = { NULL, 0, 0, QFONT_XAND, QFONT_YSHIFT, 16, 32 };
Font sfont_w = { NULL, 0, 0, SFONT_XAND, SFONT_YSHIFT, 32, 32 };
Font scorefont_b = { NULL, 0, 0, SFONT_XAND, SFONT_YSHIFT, 32, 48 }; 
Font scorefont_g = { NULL, 0, 0, SFONT_XAND, SFONT_YSHIFT, 32, 48 };


//-------------------------------------------------------------------------
// nahrava fonty
//-------------------------------------------------------------------------
void loadFonts()
{
   R16Header *header;
   qfont_r.data = surfaceManager->load("qfontr.r16","data\\pictures.dir",true,
      prefferedCAPS);
   if (qfont_r.data != NULL)
   {
      header = surfaceManager->getHeader(qfont_r.data);
      if (header != NULL)
      {
         qfont_r.xsize = header->dwCelWidth;
         qfont_r.ysize = header->dwCelHeight;
         qfont_r.rsize = header->byCelRow;
//         qfont_r.first = header->byFontFirst;
      }
      else
         throw new BubException("loadFonts: failed to load font qfontr.r16!");
   }
   else
      throw new BubException("loadFonts: failed to load font qfontr.r16!");

   qfont_g.data = surfaceManager->load("qfontg.r16","data\\pictures.dir",true,
      prefferedCAPS);
   if (qfont_g.data != NULL)
   {
      header = surfaceManager->getHeader(qfont_g.data);
      if (header != NULL)
      {
         qfont_g.xsize = header->dwCelWidth;
         qfont_g.ysize = header->dwCelHeight;
         qfont_g.rsize = header->byCelRow;
//         qfont_g.first = header->byFontFirst;
      }
      else
         throw new BubException("loadFonts: failed to load font qfontg.r16!");
   }
   else
      throw new BubException("loadFonts: failed to load font qfontg.r16!");

   qfont_b.data = surfaceManager->load("qfontb.r16","data\\pictures.dir",true,
      prefferedCAPS);
   if (qfont_b.data != NULL)
   {
      header = surfaceManager->getHeader(qfont_b.data);
      if (header != NULL)
      {
         qfont_b.xsize = header->dwCelWidth;
         qfont_b.ysize = header->dwCelHeight;
         qfont_b.rsize = header->byCelRow;
//         qfont_b.first = header->byFontFirst;
      }
      else
         throw new BubException("loadFonts: failed to load font qfontb.r16!");
   }
   else
      throw new BubException("loadFonts: failed to load font qfontb.r16!");

   qfont_y.data = surfaceManager->load("qfonty.r16","data\\pictures.dir",true,
      prefferedCAPS);
   if (qfont_y.data != NULL)
   {
      header = surfaceManager->getHeader(qfont_y.data);
      if (header != NULL)
      {
         qfont_y.xsize = header->dwCelWidth;
         qfont_y.ysize = header->dwCelHeight;
         qfont_y.rsize = header->byCelRow;
//         qfont_y.first = header->byFontFirst;
      }
      else
         throw new BubException("loadFonts: failed to load font qfonty.r16!");
   }
   else
      throw new BubException("loadFonts: failed to load font qfonty.r16!");

   qfont_w.data = surfaceManager->load("qfontbw.r16","data\\pictures.dir",true,
      prefferedCAPS);
   if (qfont_w.data != NULL)
   {
      header = surfaceManager->getHeader(qfont_w.data);
      if (header != NULL)
      {
         qfont_w.xsize = header->dwCelWidth;
         qfont_w.ysize = header->dwCelHeight;
         qfont_w.rsize = header->byCelRow;
//         qfont_w.first = header->byFontFirst;
      }
      else
         throw new BubException("loadFonts: failed to load font qfontbw.r16!");
   }
   else
      throw new BubException("loadFonts: failed to load font qfontbw.r16!");

   sfont_w.data = surfaceManager->load("smallfnt2.r16","data\\pictures.dir",true,
      prefferedCAPS);
   if (sfont_w.data != NULL)
   {
      header = surfaceManager->getHeader(sfont_w.data);
      if (header != NULL)
      {
         sfont_w.xsize = header->dwCelWidth;
         sfont_w.ysize = header->dwCelHeight;
         sfont_w.rsize = header->byCelRow;
//         sfont_w.first = header->byFontFirst;
      }
      else
         throw new BubException("loadFonts: failed to load font smallfnt2.r16!");
   }
   else
      throw new BubException("loadFonts: failed to load font smallfnt2.r16!");

   scorefont_b.data = surfaceManager->load("scoreblue.r16","data\\pictures.dir",true,
      DDSCAPS_OFFSCREENPLAIN+DDSCAPS_SYSTEMMEMORY);
   if (scorefont_b.data != NULL)
   {
      header = surfaceManager->getHeader(scorefont_b.data);
      if (header != NULL)
      {
         scorefont_b.xsize = header->dwCelWidth;
         scorefont_b.ysize = header->dwCelHeight;
         scorefont_b.rsize = header->byCelRow;
//         scorefont_b.first = header->byFontFirst;
      }
      else
         throw new BubException("loadFonts: failed to load font scoreblue.r16!");
   }
   else
      throw new BubException("loadFonts: failed to load font scoreblue.r16!");

   scorefont_g.data = surfaceManager->load("scoregreen.r16","data\\pictures.dir",true,
      prefferedCAPS);
   if (scorefont_g.data != NULL)
   {
      header = surfaceManager->getHeader(scorefont_g.data);
      if (header != NULL)
      {
         scorefont_g.xsize = header->dwCelWidth;
         scorefont_g.ysize = header->dwCelHeight;
         scorefont_g.rsize = header->byCelRow;
//         scorefont_g.first = header->byFontFirst;
      }
      else
         throw new BubException("loadFonts: failed to load font scoregreen.r16!");
   }
   else
      throw new BubException("loadFonts: failed to load font scoregreen.r16!");

}

//-------------------------------------------------------------------------
// putne text na zadanou pozici ciloveho surface
//-------------------------------------------------------------------------
void putString(DWORD x,DWORD y,Surface dest,const char *text,Font &font, 
               bool checkbounds)
{
	DDSURFACEDESC2 ddsd2;
	Surface surf;
	int textLen = strlen(text);
	DWORD textWidth = textLen * font.xsize;
   DWORD fx, fy;
   BYTE frx, fry, ff;

   if (textLen <= 0)
      return;

   fx = font.xsize;
   fy = font.ysize;
   frx = font.xand;
   fry = font.yshift;
   ff = font.first;

	if (checkbounds && (x + textWidth > 640 || y > 480-font.ysize))
   {
      sprintf(exc_string,"putString: text(%s) out of bounds!",text);
		throw new BubException(exc_string);
   }
	ZeroMemory(&ddsd2,sizeof(ddsd2));
	ddsd2.dwSize = sizeof(ddsd2);
	ddsd2.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd2.dwHeight = font.ysize;
	ddsd2.dwWidth = textWidth;
	ddsd2.ddsCaps.dwCaps = prefferedCAPS;

	ddResult = lpDD4->CreateSurface(&ddsd2,&surf,NULL);
	if (ddResult != DD_OK)
		throw new DDException("putString: CreateSurface() failed!",ddResult);

	DDCOLORKEY ddCKey = {colorKey, colorKey};
	ddResult = surf->SetColorKey(DDCKEY_SRCBLT,&ddCKey);
	if (ddResult != DD_OK)
	{
		surf->Release();
		throw new DDException("putString: SetColorKey() failed!",ddResult);
	}

   RECT r;
   try
   {
      for (int i = 0; i < textLen; i++)
      {
         r.left = ((((BYTE)text[i])-ff) & frx)*fx;
         r.right = r.left + fx;
         r.top = ((((BYTE)text[i])-ff) >> fry)*fy;
         r.bottom = r.top + fy;
         putcel((DWORD)i*fx,0,surf,r,font.data,DDBLTFAST_NOCOLORKEY);
      }
   }
   catch (Exception *exc)
   {
      if (exc->msg != exc_string)
         strcpy(exc_string,exc->msg);
      strcat(exc_string," in putString(");
      strcat(exc_string,text);
      strcat(exc_string,")");
      exc->msg = exc_string;
      throw exc;
   }

	r.left = 0;
	r.right = textWidth;
	r.top = 0;
	r.bottom = fy;

   putcel(x,y,dest,r,surf,DDBLTFAST_SRCCOLORKEY);
	surf->Release();
}

//-------------------------------------------------------------------------
// vrati surface s putnutym textem
//-------------------------------------------------------------------------
Surface getPutString(const char *text,Font &font,RECT *surfr)
{
	DDSURFACEDESC2 ddsd2;
	Surface surf;
	int textLen = strlen(text);
	DWORD textWidth = textLen * font.xsize;
   DWORD fx, fy;
   BYTE frx, fry, ff;

   if (textLen <= 0)
      return NULL;

   fx = font.xsize;
   fy = font.ysize;
   frx = font.xand;
   fry = font.yshift;
   ff = font.first;

   if (surfr != NULL)
   {
      surfr->left = 0;
      surfr->right = textWidth;
      surfr->top = 0;
      surfr->bottom = font.ysize;
   }

	if (textWidth > 640)
		throw new BubException("getPutString: text out of bounds!");
	ZeroMemory(&ddsd2,sizeof(ddsd2));
	ddsd2.dwSize = sizeof(ddsd2);
	ddsd2.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd2.dwHeight = font.ysize;
	ddsd2.dwWidth = textWidth;
	ddsd2.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;

	ddResult = lpDD4->CreateSurface(&ddsd2,&surf,NULL);
	if (ddResult != DD_OK)
		throw new DDException("getPutString: CreateSurface() failed!",ddResult);

	DDCOLORKEY ddCKey = {colorKey, colorKey};
	ddResult = surf->SetColorKey(DDCKEY_SRCBLT,&ddCKey);
	if (ddResult != DD_OK)
	{
		surf->Release();
		throw new DDException("getPutString: SetColorKey() failed!",ddResult);
	}

   try
   {
   	RECT r;
	   for (int i = 0; i < textLen; i++)
	   {
         r.left = ((((BYTE)text[i])-ff) & frx)*fx;
         r.right = r.left + fx;
         r.top = (BYTE)text[i];
         r.top = ((((BYTE)text[i])-ff) >> fry)*fy;
         r.bottom = r.top + fy;
         putcel((DWORD)i*fx,0,surf,r,font.data,DDBLTFAST_NOCOLORKEY);
      }
   }
   catch (Exception *exc)
   {
      if (exc->msg != exc_string)
         strcpy(exc_string,exc->msg);
      strcat(exc_string," in getPutString(");
      strcat(exc_string,text);
      strcat(exc_string,")");
      exc->msg = exc_string;
      throw exc;
   }
   
   return surf;
}

//-------------------------------------------------------------------------
// putne text do ciloveho surface tak, ze x-stred textu je na zadane pozici
//-------------------------------------------------------------------------
void putStringCentered(DWORD x,DWORD y,Surface dest,const char *text, 
                       Font &font)
{
	DDSURFACEDESC2 ddsd2;
	Surface surf;
	int textLen = strlen(text);
	DWORD textWidth = textLen * font.xsize;
   DWORD fx, fy;
   BYTE frx, fry, ff;
   DWORD truex;

   if (textLen <= 0)
      return;

   fx = font.xsize;
   fy = font.ysize;
   frx = font.xand;
   fry = font.yshift;
   ff = font.first;

   truex = x - (textWidth+1) / 2;  // pokud pretece,  bude truex vetsi nez x
	if (truex > x || y > (480-font.ysize))
		throw new BubException("putString: text out of bounds!");
	ZeroMemory(&ddsd2,sizeof(ddsd2));
	ddsd2.dwSize = sizeof(ddsd2);
	ddsd2.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd2.dwHeight = font.ysize;
	ddsd2.dwWidth = textWidth;
	ddsd2.ddsCaps.dwCaps = prefferedCAPS;

	ddResult = lpDD4->CreateSurface(&ddsd2,&surf,NULL);
	if (ddResult != DD_OK)
		throw new DDException("putString: CreateSurface() failed!",ddResult);

	DDCOLORKEY ddCKey = {colorKey, colorKey};
	ddResult = surf->SetColorKey(DDCKEY_SRCBLT,&ddCKey);
	if (ddResult != DD_OK)
	{
		surf->Release();
		throw new DDException("putString: SetColorKey() failed!",ddResult);
	}

   RECT r;
   try
   {
      for (int i = 0; i < textLen; i++)
      {
         r.left = ((((BYTE)text[i])-ff) & frx)*fx;
         r.right = r.left + fx;
         r.top = ((((BYTE)text[i])-ff) >> fry)*fy;
         r.bottom = r.top + fy;
         
         putcel((DWORD)i*fx,0,surf,r,font.data,DDBLTFAST_NOCOLORKEY);
      }
   }
   catch (Exception *exc)
   {
      if (exc->msg != exc_string)
         strcpy(exc_string,exc->msg);
      strcat(exc_string," in putStringCentered(");
      strcat(exc_string,text);
      strcat(exc_string,")");
      exc->msg = exc_string;
      throw exc;
   }

	r.left = 0;
	r.right = textWidth;
	r.top = 0;
	r.bottom = fy;

   putcel(truex,y,dest,r,surf,DDBLTFAST_SRCCOLORKEY);
	surf->Release();
}
