/*
 *--------------------------------------------------------------------
 *
 * Keyboard
 *
 * Objekt reprezentujici klavesnici. Implementace pres DirectInput
 *
 * verze   0.8.2 - 4.10.1999
 * autori  Vojvo
 *
 *--------------------------------------------------------------------
 * zmeny
 *
 * 0.8.1 - oprava nedostatku v clearKey()
 * 0.0.2 - pridana metoda clearKey() a promenna lastScan
 * 0.0.1 - prvni verze

  Known bugs:
  v ascii (ani jinde?) nefunguje "/" na numerick klvesnici
  asi by to chtlo njak individuln pstup nebo co

  Druh zvltnost je, e po njakm zmatkovn zanou teba Fx
  generovat jaksi nhodn znaky - zejm pedchoz obsah bufferu.
  (Perioda je maximln 10, ale jindy to vypad i na m...?!)
  (Pristupuje se tam pres LastAscii, take nkde tam...)
  
  V. 2004
  
 *--------------------------------------------------------------------
 */

#include "cbuffer.h"
#include "exceptions.h"
#include "keyboard.h"

#define BUFFER_SIZE  32
#define SFT          255					// Shift pseudo ASCII code

DECLARE_CBUFFER(BYTE,lastASCII,lastASCII_in,lastASCII_out,lastASCII_size,LAST_SIZE)
DECLARE_CBUFFER(BYTE,lastScan,lastScan_in,lastScan_out,lastScan_size,LAST_SIZE)

HRESULT diResult = DI_OK;
Keyboard kbd;

BYTE    ASCIINames[] =          // Unshifted ASCII for scan codes
{
//       0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F
        0  ,27 ,'1','2','3','4','5','6','7','8','9','0','-','=',8  ,9  , // 0
        'q','w','e','r','t','y','u','i','o','p','[',']',13 ,0  ,'a','s', // 1
        'd','f','g','h','j','k','l',';',39 ,'`',SFT,92 ,'z','x','c','v', // 2
        'b','n','m',',','.','/' ,SFT,'*',0 ,' ',0  ,0  ,0  ,0  ,0  ,0  , // 3
        0  ,0  ,0  ,0  ,0  ,0  ,0  ,'7','8','9','-','4','5','6','+','1', // 4
        '2','3','0','.',0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  , // 5
        0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  , // 6
        0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0    // 7
},
        ShiftNames[] =           // Shifted ASCII for scan codes
{
//       0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F
        0  ,27 ,'!','@','#','$','%','^','&','*','(',')','_','+',8  ,9  ,        // 0
        'Q','W','E','R','T','Y','U','I','O','P','{','}',13 ,0  ,'A','S',        // 1
        'D','F','G','H','J','K','L',':',34 ,'~',SFT,'|','Z','X','C','V',        // 2
        'B','N','M','<','>','?',SFT,'*',0  ,' ',0  ,0  ,0  ,0  ,0  ,0  ,        // 3
        0  ,0  ,0  ,0  ,0  ,0  ,0  ,'7','8','9','-','4','5','6','+','1',        // 4
        '2','3','0','.',0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,        // 5
        0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,        // 6
        0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0           // 7
},
        AsKey[] =          // Posachovana ascii pro menu
{
//       0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F
        0  ,27 ,'1','2','3','4','5','6','7','8','9','0','-','=',247,240,        // 0
        'Q','W','E','R','T','Y','U','I','O','P','[',']',246,167,'A','S',        // 1
        'D','F','G','H','J','K','L',';',39 ,'`',SFT,92 ,'Z','X','C','V',        // 2
        'B','N','M',',','.','/' ,SFT,'*',156,' ',241,0  ,0  ,0  ,0  ,0  ,        // 3
        0  ,0  ,0  ,0  ,0  ,239,250,174,244,171,'-',243,'5',242,'+',175,  //crsr// 4
        245,172,248,249,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,        // 5
        0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,        // 6
        0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,        // 7
        0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,        // 8
        0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,246,167,0  ,0  ,        // 9
        0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,        // A
        0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,156,0  ,241,0  ,0  ,0  ,0  ,0  ,        // B
        0  ,0  ,0  ,0  ,0  ,239,250,174,244,171,0  ,243,0  ,242,0  ,175,        // C
        245,172,248,249,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,        // D
        0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,        // E
        0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,0  ,        // F
};


/*--------------------------------------------------------------------*
    Keyboard
 *--------------------------------------------------------------------*/
Keyboard::Keyboard()
{
	lpDI = NULL;
	lpDIKeyboard = NULL;
   lastScanSet = NULL;
   lastScanSet_size = 0;
   filter_lastscan = true;
}

/*--------------------------------------------------------------------*
    init
 *--------------------------------------------------------------------*/
void Keyboard::init(HINSTANCE hInst, HWND hWnd)
{
	diResult = DirectInputCreate(hInst,DIRECTINPUT_VERSION,&lpDI,NULL);
	if (diResult == DIERR_INVALIDPARAM)
		throw new DIException("Keyboard::init: DirectInputCreate() invalid params!",diResult);
	if (diResult == DIERR_OUTOFMEMORY)
		throw new DIException("Keyboard::init: DirectInputCreate() out of memory!",diResult);
	if (diResult == DIERR_OLDDIRECTINPUTVERSION)
		throw new DIException("Keyboard::init: DirectInputCreate() old DirectInput version!",diResult);
	if (diResult != DI_OK)
		throw new DIException("Keyboard::init: DirectInputCreate() failed!",diResult);
	
	diResult = lpDI->CreateDevice(GUID_SysKeyboard, &lpDIKeyboard, NULL);
	if (diResult != DI_OK)
		throw new DIException("Keyboard::init: CreateDevice() failed!",diResult);

	diResult = lpDIKeyboard->SetDataFormat(&c_dfDIKeyboard);
	if (diResult != DI_OK)
		throw new DIException("Keyboard::init: SetDataFormat() failed!",diResult);

	diResult = lpDIKeyboard->SetCooperativeLevel(hWnd,
		DISCL_FOREGROUND | DISCL_NONEXCLUSIVE);
	if (diResult != DI_OK)
		throw new DIException("Keyboard::init: SetCooperativeLevel() failed!",diResult);

   DIPROPDWORD dipdw;

   dipdw.diph.dwSize = sizeof(DIPROPDWORD);
   dipdw.diph.dwHeaderSize = sizeof(DIPROPHEADER);
   dipdw.diph.dwObj = 0;
   dipdw.diph.dwHow = DIPH_DEVICE;
   dipdw.dwData = BUFFER_SIZE;

   diResult = lpDIKeyboard->SetProperty( DIPROP_BUFFERSIZE, &dipdw.diph );
	if (diResult != DI_OK)
		throw new DIException("Keyboard::init: SetProperty() failed!",diResult);

   for(int i=0; i < 256; i++)
   {
      cleared[i] = 255;
      diKeys[i] = 0;
   }
}

//-------------------------------------------------------------------------
// nastavuje klavesy odchytavane pres lastScan
//-------------------------------------------------------------------------
void Keyboard::setLastScanSet(BYTE *set, int size)
{
   lastScanSet = set;
   lastScanSet_size = size;
}

/*--------------------------------------------------------------------*
    acquire
 *--------------------------------------------------------------------*/
void Keyboard::acquire()
{
	if (lpDIKeyboard)
		lpDIKeyboard->Acquire();
}

/*--------------------------------------------------------------------*
    unacquire
 *--------------------------------------------------------------------*/
void Keyboard::unacquire()
{
	if (lpDIKeyboard)
		lpDIKeyboard->Unacquire();
}

//--------------------------------------------------------------------
//    getState
//--------------------------------------------------------------------
void Keyboard::getState()
{
   DIDEVICEOBJECTDATA didod[ BUFFER_SIZE ];  // Receives buffered data 
   DWORD dwElements;
   diResult = DIERR_INPUTLOST;

   if (lpDIKeyboard == NULL)
      throw new BubException("Keyboard::getState: NULL encountered");

   while(diResult != DI_OK)
   {
      dwElements = BUFFER_SIZE;
      diResult = lpDIKeyboard->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
                                             didod,&dwElements,0);
      if (diResult != DI_OK) 
      {
			// We got an error or we got DI_BUFFEROVERFLOW.
			//
			// Either way, it means that continuous contact with the
			// device has been lost, either due to an external
			// interruption, or because the buffer overflowed
			// and some events were lost.
			//
			// Consequently, if a button was pressed at the time
			// the buffer overflowed or the connection was broken,
			// the corresponding "up" message might have been lost.
			//
			// But since our procedure reads keyborad state after
			// processing of events there is no need to fear. 
         diResult = lpDIKeyboard->Acquire();
         if (diResult != DI_OK)  
            return;
			diResult = DIERR_INPUTLOST;
      }
	}

	// Reading of real keyboard state
	diResult = lpDIKeyboard->GetDeviceState(sizeof(diKeys),(LPVOID)&diKeys);
	if (diResult != DI_OK)
		throw new DIException("Keyboard::getState: GetDeviceState() failed!",diResult);
	
   // Reenable depressed cleared keys and apply mask of cleared keys
   for(int j=0; j < 256; j+=4)
   {
      *((DWORD*)(cleared+j)) |= ~(*((DWORD*)(diKeys+j)));
      *((DWORD*)(diKeys+j)) &= *((DWORD*)(cleared+j));
   }

   // Study each of the buffer elements and process them.
   // our "processing" consists merely of determining lastASCII
	char c;
   BYTE key;
   for (DWORD i = 0; i < dwElements; i++) 
   {
      key = (BYTE)didod[i].dwOfs;
		if (ASCIINames[key] == SFT)
		{
			shiftState = didod[i].dwData & 0x80;
			continue;
		}
      if (didod[i].dwData & 0x80)
		{
         if (key < 0x80)
         {
			   if (shiftState)
   				c = ShiftNames[key];
			   else
				   c = ASCIINames[key];
			   if (c != 0)
			   CBUFFER_INSERT(lastASCII,lastASCII_in,lastASCII_out,lastASCII_size,c)
         }
         if (filter_lastscan)
         {
		      for(int k=0; k<lastScanSet_size; k++)
            {
               if (key == lastScanSet[k])
               {
                  CBUFFER_INSERT(lastScan,lastScan_in,lastScan_out,lastScan_size,key);
	            }
            }
         }
         else
            CBUFFER_INSERT(lastScan,lastScan_in,lastScan_out,lastScan_size,key);
      }
   }
}

//-------------------------------------------------------------------------
// programove zrusi stisk klavesy a zakaze jej dokud neni uvolnena a znovu
// stistena
//-------------------------------------------------------------------------
void Keyboard::clearKey(BYTE key)
{
   diKeys[key] = 0;
   cleared[key] = 0;
}

//-------------------------------------------------------------------------
// vrati posledni stisknutou 'ASCII' klavesu
//-------------------------------------------------------------------------
BYTE Keyboard::LastASCII()
{
   BYTE result;
   
   CBUFFER_RETRIEVE(lastASCII,lastASCII_in,lastASCII_out,lastASCII_size,
      result,0);
   return result;
}

//-------------------------------------------------------------------------
//  vrati posledni stisknutou klavesu z pole sledovanych klaves
//-------------------------------------------------------------------------
BYTE Keyboard::LastScan()
{
   BYTE result;

   CBUFFER_RETRIEVE(lastScan,lastScan_in,lastScan_out,lastScan_size,
      result,0);
   return result;
}

//-------------------------------------------------------------------------
// resetuje buffer poslednich scankodu
//-------------------------------------------------------------------------
void Keyboard::lastScanReset()
{
   CBUFFER_RESET(lastScan,lastScan_in,lastScan_out);
}

//-------------------------------------------------------------------------
// resetuje buffer poslednich ASCII kodu
//-------------------------------------------------------------------------
void Keyboard::lastASCIIReset()
{
   CBUFFER_RESET(lastASCII,lastASCII_in,lastASCII_out);
}

//-------------------------------------------------------------------------
// zrusi klavesnici
//-------------------------------------------------------------------------
void Keyboard::done()
{
    // Unacquire and release any DirectInputDevice objects.
    if (lpDIKeyboard != NULL) 
    {
        // Unacquire the device one last time just in case 
        // the app tried to exit while the device is still acquired.
        lpDIKeyboard->Unacquire();

        lpDIKeyboard->Release();
        lpDIKeyboard = NULL;
    }

    // Release any DirectInput objects.
    if (lpDI != NULL) 
    {
        lpDI->Release();
        lpDI = NULL;
    }
}

//-------------------------------------------------------------------------
// destruktor
//-------------------------------------------------------------------------
Keyboard::~Keyboard()
{
	done();
}




