/*
  R16 graphic format
  Written by Vojvo & Gunny
  Version 0.99.3

 8.12. - uprava loadovani struktur
 1. 8. - predelane exceptions
31. 7. / opravena chyba pri dekompresi na 565
25. 7. / mela by chodit dekomprese (netestovano) 

*/

#include <stdio.h>
#include "fakedir.h"
#include "graphDD.h"
#include "exceptions.h"
#include "utils.h"
#include "r16.h"

const char R16ID[4] = "002";
const char R16DAT[4] = "DAT";

//----------------------------------------------------------------------------
// saveR16
//----------------------------------------------------------------------------
// uklada obsah surface do souboru R16
void saveR16(Surface surf, TCHAR *name, BYTE gbits, DWORD celx, DWORD cely,
             BYTE celrow, BYTE fontfirst)
{
	FILE *output;
	long lPitch;
	BYTE *lpSurface;
   R16Header header;

   // otevreni streamu
   output = fopen(name,"wb");
   if (!output)
   {
      throw new BubException("saveR16: failed to create file!");
   }

   // priprava hlavicky R16 - 1.etapa
   strcpy(header.szId,R16ID);
   strcpy(header.szStartData,R16DAT);
   header.dwCelWidth = celx;
   header.dwCelHeight = cely;
   header.byCelRow = celrow;
   header.byGreenBits = gbits;
   header.byFontFirst = fontfirst;

   for(int ii=0; ii<R16_RESERVED_CNT; ii++)
      header.reserved[ii] = 0;

	// lock surface
   DDSURFACEDESC2 ddsd2;
	lock(surf,NULL,&ddsd2);

   // priprava hlavicky R16 - 2.etapa
	header.dwWidth = ddsd2.dwWidth;
	header.dwHeight = ddsd2.dwHeight;
   
   // zapis hlavicky
   if (fwrite(&header,sizeof(header),1,output) != 1)
   {
      fclose(output);
      remove(name);
      throw new BubException("saveR16: failed to write file");
   }
   
   // zapis dat surface
   lPitch = ddsd2.lPitch;
	lpSurface = static_cast<BYTE *>(ddsd2.lpSurface);

	for (DWORD i = 0; i < header.dwHeight; i++)
		if (fwrite(lpSurface+i*lPitch,header.dwWidth*sizeof(WORD),1,output) != 1)
      {
         fclose(output);
         remove(name);
         throw new BubException("saveR16: failed to write file");
      }

   // unlock surface
	surf->Unlock(NULL);
	fclose(output);
}

//----------------------------------------------------------------------------
// reLoadR16
//----------------------------------------------------------------------------
// nahrava R16 do existujici surface

// priprava =)
#define PLACEPIXEL               \
{ WORD pixval; \
if (!bGBitsLoaded)               \
  pixval = TO565(that);          \
else                             \
  pixval = that;                 \
  *pos = pixval;                 \
  linepos++;                     \
  if ((DWORD)linepos >= dwWidth) \
  {                              \
     linepos = 0;                \
     pos += linedif;             \
  }                              \
  pos++;                         \
i++;}

void reLoadR16(Surface surf, char *name, char *fakename, R16Header *hdr)
{
	DDSURFACEDESC2 ddsd2;
	WORD *lpSurface;
	long lPitch;
   bool bGBitsLoaded;
	FILE *input;
   DWORD dwWidth, dwHeight;
   R16Header header;

   // open file
   input = fakeopen(fakename,name,"rb");
   if (input == NULL)
   {
      sprintf(exc_string,"reLoadR16: fakeopen(%s) failed!",name);
      throw new BubException(exc_string);
   }

   // nacteni hlavicky
   fread(&(header.szId),sizeof(header.szId),1,input);
   fread(&(header.dwWidth),sizeof(DWORD),1,input);
   fread(&(header.dwHeight),sizeof(DWORD),1,input);
   fread(&(header.dwCelWidth),sizeof(DWORD),1,input);
   fread(&(header.dwCelHeight),sizeof(DWORD),1,input);
   fread(&(header.byCelRow),sizeof(BYTE),1,input);
   fread(&(header.byGreenBits),sizeof(BYTE),1,input);
   fread(&(header.byType),sizeof(BYTE),1,input);
   fread(&(header.byFontFirst),sizeof(BYTE),1,input);
   fread(&(header.reserved),sizeof(BYTE),R16_RESERVED_CNT,input);
   fread(&(header.szStartData),sizeof(header.szStartData),1,input);
	if (strcmp(header.szId,R16ID))
   {
      fclose(input);
      sprintf(exc_string,"reLoadR16: %s noncompatible format!",name);
		throw new BubException(exc_string);
   }

   if (hdr != NULL)
   {
      strcpy(hdr->szId,header.szId);
      hdr->dwWidth = header.dwWidth;
      hdr->dwHeight = header.dwHeight;
      hdr->dwCelWidth = header.dwCelWidth;
      hdr->dwCelHeight = header.dwCelHeight;
      hdr->byCelRow = header.byCelRow;
      hdr->byGreenBits = header.byGreenBits;
      hdr->byType = header.byType;
      hdr->byFontFirst = header.byFontFirst;
   }

	// zamceni surface
   lock(surf,NULL,&ddsd2);

	lpSurface = (WORD*) ddsd2.lpSurface;
	dwWidth = ddsd2.dwWidth;
	dwHeight = ddsd2.dwHeight;
	lPitch = ddsd2.lPitch / sizeof(WORD);

   // test velikosti surface
   if (dwWidth < header.dwWidth || dwHeight < header.dwHeight)
	{
		surf->Unlock(NULL);
      fclose(input);
      sprintf(exc_string,"reLoadR16: %s noncompatible dimensions!",name);
		throw new BubException(exc_string);
	}


   // zjisteni souhlasu nactenych a skutecnych zelenych bitu
   if ((int)header.byGreenBits == gBits)
     bGBitsLoaded = true;
   else
     bGBitsLoaded = false;

   // jak to vypada s formatem?
   if (header.byType == 2)
   {
     // KOMPRIMOVANE!!!

     WORD * pos = lpSurface;
     // tak ted ta dekomprese!
     int size = (header.dwWidth)*(header.dwHeight);

     // jedeme skrz
     BYTE wr;
     WORD last = 0;
     WORD that = 0;
     WORD dif;
     BYTE rep;
     //
     int dr,dg;
     int i = 0;
     long linepos = 0;
     long linedif = (lPitch - header.dwWidth);
     while (i<size)
     {
       // cteni ze souboru
       if (fread(&wr,1,1,input)==1)
       {
         if (wr&0x80)
         {
           // specialni znak
           if (wr&0x3f)
           {
             // je to relativni rozdil
             dr = (wr&0x30);
             dg = (wr&0x0c);
             dif = (wr&0x03);
             dif |= (dg<<3);
             dif |= (dr<<6);
             // kterym smerem se rozdil pocita?
             if (wr&0x40)
             {
               // minus
               that -= dif;
             }
             else
             {
               // plus
               that += dif;
             }
             // zapis do surfacu
             PLACEPIXEL
           }
           else
           {
             // je to stejny pixel
             // zapis do souboru
             PLACEPIXEL
             // je tam i nejake opakovani?
             if (wr&0x40)
             {
               // opakuje se vicekrat
               fread(&rep,1,1,input); //(wr&0x3f);
               while (rep>0)
               {
                 // zapis do surfacu
                 PLACEPIXEL
                 rep--;
               }
             }
           }
         }
         else
         {
           // proste pixel
           that = wr;
           that = (that << 8);
           fread(&wr,1,1,input);
           that = (that | wr);
           // zapis do surfacu
           PLACEPIXEL
         }
       }
       else
       {
         // krizka - cosi nevyslo
         that = 0x007f;
         // zapis do surface
         PLACEPIXEL
       }
     }
   }
   else
   {
     // NEKOMPRIMOVANE
 
     for(DWORD i = 0; i < header.dwHeight; i++)
     {
       if (fread(&(lpSurface[i*lPitch]),sizeof(WORD),header.dwWidth,
         input) < header.dwWidth)
       {
         surf->Unlock(NULL);
         fclose(input);
         sprintf(exc_string,"reLoadR16: %s read error!",name);
	   	throw new BubException(exc_string);
         
       }
       if (!bGBitsLoaded)
       {
         DWORD sOff = i*lPitch;
         for(DWORD j = 0; j < header.dwWidth; j++)
           if (gBits == 5)
             lpSurface[sOff+j] = TO555(lpSurface[sOff+j]);
           else
             lpSurface[sOff+j] = TO565(lpSurface[sOff+j]);
       }
     }
   }

	fclose(input);
	surf->Unlock(NULL);
}

//----------------------------------------------------------------------------
// nahrava R16 do nove surface
//----------------------------------------------------------------------------
Surface loadR16(char *name, char *fakename, R16Header *hdr, DWORD dwFlags)
{
	Surface surf;
	DDSURFACEDESC2 ddsd2;
	WORD *lpSurface;
	DWORD dwWidth, dwHeight;
	long lPitch;
	FILE *input;
	BYTE bGBitsLoaded;
   R16Header header;
	
   if (name == NULL)
      throw new BubException("loadR16: NULL!");
   
   // open file
   input = fakeopen(fakename,name,"rb");
   if (input == NULL)
   {
      sprintf(exc_string,"loadR16: fakeopen(%s) failed!",name);
      throw new BubException(exc_string);
   }
   
   // nacteni hlavicky
   fread(&(header.szId),sizeof(header.szId),1,input);
   fread(&(header.dwWidth),sizeof(DWORD),1,input);
   fread(&(header.dwHeight),sizeof(DWORD),1,input);
   fread(&(header.dwCelWidth),sizeof(DWORD),1,input);
   fread(&(header.dwCelHeight),sizeof(DWORD),1,input);
   fread(&(header.byCelRow),sizeof(BYTE),1,input);
   fread(&(header.byGreenBits),sizeof(BYTE),1,input);
   fread(&(header.byType),sizeof(BYTE),1,input);
   fread(&(header.byFontFirst),sizeof(BYTE),1,input);
   fread(&(header.reserved),sizeof(BYTE),R16_RESERVED_CNT,input);
   fread(&(header.szStartData),sizeof(header.szStartData),1,input);
	if (strcmp(header.szId,R16ID))
   {
      fclose(input);
      sprintf(exc_string,"loadR16: %s noncompatible format!",name);
		throw new BubException(exc_string);
   }

   if (hdr != NULL)
   {
      strcpy(hdr->szId,header.szId);
      hdr->dwWidth = header.dwWidth;
      hdr->dwHeight = header.dwHeight;
      hdr->dwCelWidth = header.dwCelWidth;
      hdr->dwCelHeight = header.dwCelHeight;
      hdr->byCelRow = header.byCelRow;
      hdr->byGreenBits = header.byGreenBits;
      hdr->byFontFirst = header.byFontFirst;
   }

   // vytvoreni surface
	ZeroMemory(&ddsd2,sizeof(ddsd2));
	ddsd2.dwSize = sizeof(ddsd2);
	ddsd2.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
	ddsd2.dwWidth = header.dwWidth;
	ddsd2.dwHeight = header.dwHeight;
	ddsd2.ddsCaps.dwCaps = dwFlags;
   if (header.dwWidth > 640)
      ddsd2.ddsCaps.dwCaps += DDSCAPS_SYSTEMMEMORY;

	ddResult = lpDD4->CreateSurface(&ddsd2,&surf,NULL);
	if (ddResult != DD_OK)
   {
      fclose(input);
		throw new DDException("loadR16: CreateSurface() failed!",ddResult);
   }

	DDCOLORKEY ddCKey;
	ddCKey.dwColorSpaceLowValue = (DWORD) colorKey;
	ddCKey.dwColorSpaceHighValue = (DWORD) colorKey;
	ddResult = surf->SetColorKey(DDCKEY_SRCBLT,&ddCKey);
	if (ddResult != DD_OK)
   {
      fclose(input);
		throw new DDException("loadR16: SetColorKey() failed!",ddResult);
   }

	lock(surf,NULL,&ddsd2);

	lpSurface = (WORD*) ddsd2.lpSurface;
	dwWidth = ddsd2.dwWidth;
	dwHeight = ddsd2.dwHeight;
	lPitch = ddsd2.lPitch / sizeof(WORD);

	if ((int)header.byGreenBits == gBits)
		bGBitsLoaded = true;
   else
		bGBitsLoaded = false;

   // jak to vypada s formatem?
   if (header.byType == 2)
   {
     // KOMPRIMOVANE!!!

     WORD * pos = lpSurface;
     // tak ted ta dekomprese!
     int size = (header.dwWidth)*(header.dwHeight);

     // jedeme skrz
     BYTE wr;
     WORD last = 0;
     WORD that = 0;
     WORD dif;
     BYTE rep;
     //
     int dr,dg;
     int i = 0;
     long linepos = 0;
     long linedif = (lPitch - header.dwWidth);
     while (i<size)
     {
       // cteni ze souboru
       if (fread(&wr,1,1,input)==1)
       {
         if (wr&0x80)
         {
           // specialni znak
           if (wr&0x3f)
           {
             // je to relativni rozdil
             dr = (wr&0x30);
             dg = (wr&0x0c);
             dif = (wr&0x03);
             dif |= (dg<<3);
             dif |= (dr<<6);
             // kterym smerem se rozdil pocita?
             if (wr&0x40)
             {
               // minus
               that -= dif;
             }
             else
             {
               // plus
               that += dif;
             }
             // zapis do surfacu
             PLACEPIXEL
           }
           else
           {
             // je to stejny pixel
             // zapis do souboru
             PLACEPIXEL
             // je tam i nejake opakovani?
             if (wr&0x40)
             {
               // opakuje se vicekrat
               fread(&rep,1,1,input); //(wr&0x3f);
               while (rep>0)
               {
                 // zapis do surfacu
                 PLACEPIXEL
                 rep--;
               }
             }
           }
         }
         else
         {
           // proste pixel
           that = wr;
           that = (that << 8);
           fread(&wr,1,1,input);
           that = (that | wr);
           // zapis do surfacu
           PLACEPIXEL
         }
       }
       else
       {
         // krizka - cosi nevyslo
         that = 0x007f;
         // zapis do surface
         PLACEPIXEL
       }
     }
   }
   else
   {
      // NEKOMPRIMOVANE
      
      for(DWORD i = 0; i < dwHeight; i++)
      {
         if (fread(&(lpSurface[i*lPitch]),sizeof(WORD),dwWidth,input)<dwWidth)
         {
            surf->Unlock(NULL);
            fclose(input);
            sprintf(exc_string,"loadR16: %s read error!",name);
		      throw new BubException(exc_string);
         }
         if (!bGBitsLoaded)
         {
            DWORD sOff = i*lPitch;
            for(DWORD j = 0; j < dwWidth; j++)
               if (gBits == 5)
                  lpSurface[sOff+j] = TO555(lpSurface[sOff+j]);
               else
                  lpSurface[sOff+j] = TO565(lpSurface[sOff+j]);
         }
      }
	}

	fclose(input);
	surf->Unlock(NULL);
	return surf;
}

//-------------------------------------------------------------------------
// nahrava R16 jako pole WORDu
//-------------------------------------------------------------------------
WORD * loadR16WORD(char *name, char *fakename, R16Header *hdr)
{
   R16Header header;
   WORD *r16;
   DWORD size;
   FILE *input;
	
   // open file
   input = fakeopen(fakename,name,"rb");
   if (input == NULL)
      throw new BubException("loadR16WORD: f(ake)open() failed!");	
   
   // nacteni hlavicky
   fread(&(header.szId),sizeof(header.szId),1,input);
   fread(&(header.dwWidth),sizeof(DWORD),1,input);
   fread(&(header.dwHeight),sizeof(DWORD),1,input);
   fread(&(header.dwCelWidth),sizeof(DWORD),1,input);
   fread(&(header.dwCelHeight),sizeof(DWORD),1,input);
   fread(&(header.byCelRow),sizeof(BYTE),1,input);
   fread(&(header.byGreenBits),sizeof(BYTE),1,input);
   fread(&(header.byType),sizeof(BYTE),1,input);
   fread(&(header.byFontFirst),sizeof(BYTE),1,input);
   fread(&(header.reserved),sizeof(BYTE),R16_RESERVED_CNT,input);
   fread(&(header.szStartData),sizeof(header.szStartData),1,input);
   if (strcmp(header.szId,R16ID))
   {
      fclose(input);
      throw new BubException("loadR16WORD: noncompatible format!");
   }

   if (hdr != NULL)
   {
      strcpy(hdr->szId,header.szId);
      hdr->dwWidth = header.dwWidth;
      hdr->dwHeight = header.dwHeight;
      hdr->dwCelWidth = header.dwCelWidth;
      hdr->dwCelHeight = header.dwCelHeight;
      hdr->byCelRow = header.byCelRow;
      hdr->byGreenBits = header.byGreenBits;
   }

   size = header.dwWidth * header.dwHeight;
   r16 = new WORD[size];
   if (fread(r16,sizeof(WORD),size,input) < size)
   {
      fclose(input);
      throw new BubException("loadR16WORD: read error!");
   }

   fclose(input);

   return r16;
}

//----------------------------------------------------------------------------
// uklada obsah pole WORDU a hlavicku jako soubor R16
//----------------------------------------------------------------------------
void saveR16WORD(WORD *r16, char *name, R16Header *hdr)
{
	FILE *output;
   DWORD size;

   // otevreni streamu
   output = fopen(name,"wb");
   if (!output)
   {
      throw new BubException("saveR16WORD: failed to create file!");
   }

   // zapis hlavicky
   if (fwrite(hdr,sizeof(*hdr),1,output) != 1)
   {
      fclose(output);
      remove(name);
      throw new BubException("saveR16WORD: failed to write file");
   }
   
   // zapis dat
   size = hdr->dwWidth * hdr->dwHeight;
   if (fwrite(r16,size*sizeof(WORD),1,output) != 1)
   {
      fclose(output);
      remove(name);
      throw new BubException("saveR16WORD: failed to write file");
   }

   // unlock surface
	fclose(output);
}
