/*
 *--------------------------------------------------------------------
 *
 * SurfaceManager
 *
 * Spravuje surfaces. Kazda surface je prirazena spolu se svym
 * jmenem - pokud nejde o jmeno souboru, je prvnim znakem jmena
 * hvezdicka
 *
 * verze   0.7.1 - 29.09.1999
 * autori  Vojvo
 *
 *--------------------------------------------------------------------
 * zmeny
 *
 * 0.7.1 - pri duplicitnim jmenu se v msg vyjimky toto jmeno objevi
 * 0.07 - predelano na pouziti univerzalniho fakeopenu (open|fake)
 * 0.06 - do load a restore pridano nepouzivani directory, kdyz je NULL
 * 0.05 - pridana metoda createSurface
 * 0.04 - pri nahravani je mozne predat pointer na R16header, ktery
 *        se naplni informacema o nahrane grafice
 * 0.03 - nahravani predelano na pouziti loadR16 a reLoadR16
 * 0.02 - pridana globalni promenna 'surfaceManager'
 * 0.01 - prvni verze
 *--------------------------------------------------------------------
 */
#include <string.h>
#include <stdio.h>
#include "fakedir.h"
#include "graphdd.h"
#include "exceptions.h"
#include "r16.h"
#include "utils.h"
#include "surfacemanager.h"

SurfaceManager *surfaceManager = NULL;

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
SurfaceManager::SurfaceManager()
{
   count = 0;
   directory = NULL;
}

//-------------------------------------------------------------------------
// nastavuje adresar pro nacitani souboru
//-------------------------------------------------------------------------
void SurfaceManager::setDirectory(char *dir)
{
   if (directory != NULL)
   {
      delete [] directory;
      directory = NULL;
   }
   directory = new char[strlen(dir)+2];
   directory[0] = '\x0';
   strcpy(directory,dir);
   int len = strlen(directory);
   if (len > 0 && directory[len-1] != '\\')
      strcat(directory,"\\");
}

//-------------------------------------------------------------------------
// prida zadanou surface pod specifikovanym jemenem
//-------------------------------------------------------------------------
void SurfaceManager::add(Surface surf, char *name, char *fakename, 
                         R16Header *r16h, bool useDirectory)
{
   int i = 0;

   while (i < count)
   {
      if (!strcmp(surfaces[i].szName,name))
         if (surf != surfaces[i].lpDDSData)
         {
            sprintf(exc_string,"SurfaceManager.Add(%s): Duplicit surface name",name);
            throw new BubException(exc_string);
         }
         else
            return;
      i++;
   }

   if (count < MAX_SURFACES)
   {
      if (name != NULL)
      {
         surfaces[count].szName = new char[strlen(name)+1];
         strcpy(surfaces[count].szName,name);
      }
      else
         surfaces[count].szName = NULL;
      if (fakename != NULL)
      {
         surfaces[count].szFakename = new char[strlen(fakename)+1];   
         strcpy(surfaces[count].szFakename,fakename);
      }
      else
         surfaces[count].szFakename = NULL;
      surfaces[count].lpDDSData = surf;
      surfaces[count].lpR16h = r16h;
      surfaces[count].bUseDirectory = useDirectory;
      count++;
   }
   else
      throw new BubException("SurfaceManager.Add: Too many surfaces");
}

//-------------------------------------------------------------------------
// vytvori novy prazdny surface
//-------------------------------------------------------------------------
Surface SurfaceManager::create(DWORD width, DWORD height, char *name, 
      DWORD caps)
{
   DDSURFACEDESC2 ddsd2;
   Surface surf;

   ZeroMemory(&ddsd2,sizeof(ddsd2));
   ddsd2.dwSize = sizeof(ddsd2);
   ddsd2.dwFlags = DDSD_HEIGHT | DDSD_WIDTH | DDSD_CAPS;
   ddsd2.dwWidth = width;
   ddsd2.dwHeight = height;
   ddsd2.ddsCaps.dwCaps = caps;

   ddResult = lpDD4->CreateSurface(&ddsd2, &surf, NULL);
   if (ddResult != DD_OK)
   {
      throw new DDException("create: CreateSurface() failed!",ddResult);
   }

   add(surf, name, NULL, NULL, false);
   return surf;
}

//-------------------------------------------------------------------------
// obnovi surfaces, ktere maji zadane jmeno souboru, ze ktereho jsou 
// nahrany (ty ktere nemaji prvni znak ve jmenu '*')
//-------------------------------------------------------------------------
void SurfaceManager::restore()
{
   int i = 0;

   ddRestored = true;

   while (i < count)
   {
      if (surfaces[i].lpDDSData->IsLost() == DDERR_SURFACELOST)
      {
         surfaces[i].lpDDSData->Restore();
         if (surfaces[i].szName[0] != '*')
         {
            if (surfaces[i].bUseDirectory && directory != NULL)
            {
               strcpy(pom_string,directory);
               strcat(pom_string,surfaces[i].szName);
               reLoadR16(surfaces[i].lpDDSData,pom_string,surfaces[i].szFakename,surfaces[i].lpR16h);
            }
            else
            {
               reLoadR16(surfaces[i].lpDDSData,surfaces[i].szName,
                  surfaces[i].szFakename,surfaces[i].lpR16h);
            }
         }
      }
      i++;
   }
}

//-------------------------------------------------------------------------
// nastavi jmeno pro zadany surface
//-------------------------------------------------------------------------
void SurfaceManager::setName(Surface surf, char *name)
{
   int i = 0;

   while (i < count)
   {
      if (surfaces[i].lpDDSData == surf)
      {
         delete [] surfaces[i].szName;
         surfaces[i].szName = new char[strlen(name)+1];
         strcpy(surfaces[i].szName,name);
         return;
      }
      i++;
   }
   
   return;
}


//-------------------------------------------------------------------------
// vrati surface se zadanym jmenem
//-------------------------------------------------------------------------
Surface SurfaceManager::get(char *name)
{
   int i = 0;

   while (i < count)
   {
      if (!stricmp(surfaces[i].szName,name))
         return surfaces[i].lpDDSData;
      i++;
   }
   
   return NULL;
}

//-------------------------------------------------------------------------
// vytvori novou surface se zadanym jmenem a hraje ji ze stejnojmenneho
// souboru
//-------------------------------------------------------------------------
Surface SurfaceManager::load(char *name, char *fakename, bool useDirectory, 
                             DWORD dwCaps)
{
   Surface surf;
   R16Header *r16h = new R16Header;
   int i = 0;

   while (i < count)
   {
      if (!stricmp(surfaces[i].szName,name))
         return surfaces[i].lpDDSData;
      i++;
   }

   if (useDirectory && directory != NULL)
   {
      strcpy(pom_string,directory);
      strcat(pom_string,name);
      surf = loadR16(pom_string,fakename,r16h,dwCaps);
   }
   else
   {
      surf = loadR16(name,fakename,r16h,dwCaps);
   }
   add(surf,name,fakename,r16h,useDirectory);
   return surf;
}

//-------------------------------------------------------------------------
// odstrani zadany surface z manageru
//-------------------------------------------------------------------------    
void SurfaceManager::remove(Surface surf, bool release)
{
    int i = 0;

    while (i < count)
    {
        if (surfaces[i].lpDDSData == surf)
        {
            if (release)
                surfaces[i].lpDDSData->Release();
            surfaces[i].lpDDSData = NULL;
            surfaces[i].lpDDSData = surfaces[count-1].lpDDSData;
            if (surfaces[i].szName != NULL)
               delete surfaces[i].szName;
            if (surfaces[i].szFakename != NULL)
               delete surfaces[i].szFakename;
            surfaces[i].szName = surfaces[count-1].szName;
            surfaces[i].szFakename = surfaces[count-1].szFakename;
            if (surfaces[i].lpR16h != NULL)
               delete surfaces[i].lpR16h;
            surfaces[i].lpR16h = surfaces[count-1].lpR16h;
            count--;
            return;
        }
        i++;
    }
}

//-------------------------------------------------------------------------
// odstrani surface se zadanym jmenem z manageru
//-------------------------------------------------------------------------
void SurfaceManager::remove(char *name, bool release)
{
   int i = 0;

   while (i < count)
   {
      if (!stricmp(surfaces[i].szName,name))
      {
         if (release)
            surfaces[i].lpDDSData->Release();
         surfaces[i].lpDDSData = NULL;
         surfaces[i].lpDDSData = surfaces[count-1].lpDDSData;
         if (surfaces[i].szName != NULL)
            delete surfaces[i].szName;
         if (surfaces[i].szFakename != NULL)
            delete surfaces[i].szFakename;
         surfaces[i].szName = surfaces[count-1].szName;
         surfaces[i].szFakename = surfaces[count-1].szFakename;
         if (surfaces[i].lpR16h != NULL)
            delete surfaces[i].lpR16h;
         surfaces[i].lpR16h = surfaces[count-1].lpR16h;
         count--;
         return;
      }
      i++;
   }
}

//-------------------------------------------------------------------------
// vraci ukazatel na nactenou hlavicku souboru se surfacem
//-------------------------------------------------------------------------
R16Header * SurfaceManager::getHeader(Surface surf)
{
   int i=0;

   while (i<count)
   {
      if (surfaces[i].lpDDSData == surf)
      {
         return surfaces[i].lpR16h;
      }
      i++;
   }

   return NULL;
}

//-------------------------------------------------------------------------
// vraci ukazatel na nactenou hlavicku souboru se surfacem
//-------------------------------------------------------------------------
R16Header * SurfaceManager::getHeader(char *name)
{
   int i=0;

   while (i<count)
   {
      if (!strcmp(surfaces[i].szName,name))
      {
         return surfaces[i].lpR16h;
      }
      i++;
   }

   return NULL;
}

//-------------------------------------------------------------------------
// destruktor
//-------------------------------------------------------------------------
SurfaceManager::~SurfaceManager()
{
   int i = 0;

   if (directory != NULL)
   {
      delete [] directory;
      directory = NULL;
   }

   while (i < count)
   {
      surfaces[i].lpDDSData->Release();
      if (surfaces[i].szName != NULL)
         delete surfaces[i].szName;
      if (surfaces[i].szFakename != NULL)
         delete surfaces[i].szFakename;
      if (surfaces[i].lpR16h != NULL)
         delete surfaces[i].lpR16h;
      i++;
   }
}
    