/*--------------------------------------------------------------------
 *
 * AVIPlayer
 *
 * Usek hry - prehravac animaci
 *
 * verze   0.99.4 - 6.12.1999
 * autori  Vojvo
 *
 *--------------------------------------------------------------------
 * zmeny
 * 0.99.4 - prvni verze
 */

#include <stdio.h>
#include <windows.h>
#include <vfw.h>
#include "utils.h"
#include "graphdd.h"
#include "surfaceManager.h"
#include "application.h"
#include "game.h"
#include "AVIPlayer.h"

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
AVIPlayer::AVIPlayer()
{
   aviName = NULL;
   hWndAVI = 0;
}

//-------------------------------------------------------------------------
// nastavuje parametry AVI souboru
//-------------------------------------------------------------------------
void AVIPlayer::setAVI(const char *name,int width,int height)
{
   aviName = name;
   aviWidth = width;
   aviHeight = height;
}

//-------------------------------------------------------------------------
// inicializuje veci potrebne pro spousteni
//-------------------------------------------------------------------------
void AVIPlayer::load()
{
}

//-------------------------------------------------------------------------
// zahajuje cinnost
//-------------------------------------------------------------------------
void AVIPlayer::start()
{
   DEBUG_MESSAGE("AVIPlayer::start()\n");
   if (aviName == NULL)
      return;

   FILE *ftest;
   if ((ftest = fopen(aviName,"rb")) == NULL)
   {
      aviName = NULL;
      return;
   }

   fclose(ftest);

   if (lpDD != NULL)
      doneDD();

   hWndAVI = CreateWindow(
      MCIWND_WINDOW_CLASS,
      "BubAVI",
      WS_CHILD | WS_VISIBLE,
      (app->m_rcDispArea.right-aviWidth)/2,
      (app->m_rcDispArea.bottom-aviHeight)/2,
      aviWidth,
      aviHeight,
      app->m_hWnd,
      NULL,
      app->m_hInstance,
      NULL);

   if (!hWndAVI)
   {
      aviName = NULL;
      return;
   }

   MCIWndChangeStyles(hWndAVI,
      MCIWNDF_NOMENU|MCIWNDF_NOPLAYBAR|MCIWNDF_NOERRORDLG|MCIWNDF_NOTIFYMODE,
      MCIWNDF_NOMENU|MCIWNDF_NOPLAYBAR|MCIWNDF_NOERRORDLG|MCIWNDF_NOTIFYMODE);
   if (MCIWndOpen(hWndAVI,aviName,NULL))
   {
      done = true;
      return;
   }
   MCIWndSetZoom(hWndAVI,100);
   if (MCIWndPlay(hWndAVI))
   {
      done = true;
      return;
   }

   done = false;
   DEBUG_MESSAGE("AVIPlayer::start()\n");
}

//-------------------------------------------------------------------------
// pravidelna cinnost pri kazdem pruchodu hlavni smyckou programu
//-------------------------------------------------------------------------
void AVIPlayer::update()
{
   if (aviName == NULL)
   {
      done = true;
   }

   // klavesnice
   kbd.getState();

   if (KEYDOWN(kbd.diKeys,DIK_ESCAPE))
   {
      kbd.clearKey(DIK_ESCAPE);
      done = true;
   }
   else if (KEYDOWN(kbd.diKeys,DIK_RETURN))
   {
      kbd.clearKey(DIK_RETURN);
      done = true;
   }
   else if (KEYDOWN(kbd.diKeys,DIK_SPACE))
   {
      kbd.clearKey(DIK_SPACE);
      done = true;
   }
   
   if (done)
      app->stateChange();
}

//-------------------------------------------------------------------------
// obnoveni po surface lost
//-------------------------------------------------------------------------
void AVIPlayer::restore()
{
}

//-------------------------------------------------------------------------
// cinnost pri zastaveni zobrazovani objektu
//-------------------------------------------------------------------------
void AVIPlayer::stop()
{
   DEBUG_MESSAGE("AVIPlayer::stop()\n");
   if (aviName != NULL)
   {
      if (hWndAVI)
      {
         DestroyWindow(hWndAVI);
         hWndAVI = NULL;
      }
      if (!stageError)
      {
         initDD(app->getMainWindow(),640,480,16,0);
         kbd.acquire(); 
         surfaceManager->setDirectory(app->m_szDataDir);
         gfx_load();
      }
   }
   aviName = NULL;
   DEBUG_MESSAGE("AVIPlayer::stop()\n");
}

//-------------------------------------------------------------------------
// cinnost pri definitivnim ukonceni pouzivani
//-------------------------------------------------------------------------
void AVIPlayer::unload()
{
}

