//*************************************************************************
//                         Bonus class implementation                       
//*************************************************************************
//  Written by : Vojvo (& G.)                                                  
//  Language   : VisuC6 & DX6.1
//  Version    : 0.10.1
//  Last update: 14.11.99
//-------------------------------------------------------------------------
// changes:
// 0.10.1 - zvuky stereo
// 0.8.1 - pridany zvuky


#include <stdlib.h>
#include "graphDD.h"
#include "exceptions.h"
#include "utils.h"
#include "wsound.h"
#include "game.h"
#include "sprites.h"

#define BONUS_TIME 900

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Bonus::Bonus (int onx, int ony, Property *prop) : Sprite(onx,ony,prop)
{
   cached = 0;
   cacheable = false;
   value = prop->value1;
   movex = -BONUS_MOVEX1 + random(BONUS_MOVEX2);
   movey = -BONUS_MOVEY - random(BONUS_MOVEY);
   cachedgfx = NULL;
   counter = 0;
   action = SPMA_JUMP;
   tintcolor = prop->tintcolor;
   totalxsize = prop->totxsize;
   totalysize = prop->totysize;
}

//-------------------------------------------------------------------------
// destruktor
//-------------------------------------------------------------------------
Bonus::~Bonus()
{
   SAFE_RELEASE(cachedgfx);
}

//-------------------------------------------------------------------------
// vykreslovani bonusu
//-------------------------------------------------------------------------
void Bonus::Paint(Surface dest)
{
   int y = newy;
   int ybottom = newy + ysize - 1, xright = newx + xsize - 1;
   if (newy <= PLAYFIELD_MIN_Y)
   {
      frameRect.top = frameRect.bottom - (newy - b_miny);
      y = PLAYFIELD_MIN_Y;
      clipped = true;
   }
   else if (newy >= b_maxy)
   {
      clipped = true;
      frameRect.bottom = frameRect.top + PLAYFIELD_MAX_Y-newy;
      ybottom = PLAYFIELD_MAX_Y-1;
   }
   else if (clipped)
   {
      frameRect.left = frameLeft;
      frameRect.top = frameTop;
      frameRect.right = frameLeft + xsize;  
      frameRect.bottom = frameTop + ysize;  
      clipped = false;
   }
   if (!clipped || frameRect.bottom > frameRect.top)
   {
      if ((counter >= BONUS_WARN) 
         && (counter%BONUS_BLINKPERIOD) < BONUS_BLINKLENGTH)
      {
         if (cacheable)
         {
            int framebit = 1 << frames.frameid;
            if (cached & framebit)
            {
               putcel(newx,y,dest,frameRect,cachedgfx);
            }
            else
            {
               if (cachedgfx == NULL)
               {
                  DDSURFACEDESC2 ddsd2;
                  ZeroMemory(&ddsd2,sizeof(ddsd2));
                  ddsd2.dwSize = sizeof(ddsd2);
                  ddsd2.dwWidth = totalxsize;
                  ddsd2.dwHeight = totalysize;
                  ddsd2.dwFlags = DDSD_HEIGHT | DDSD_WIDTH | DDSD_CAPS;
                  ddsd2.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;

                  ddResult = lpDD4->CreateSurface(&ddsd2,&cachedgfx,NULL);
                  if (ddResult != DD_OK)
                     throw new DDException("Bonus::Paint: CreateSurface() failed!",ddResult);
                  setColorKey(cachedgfx,colorKey);
               }
               puttransparentcached(newx,y,dest,
                  frameRect.left,frameRect.top,cachedgfx,
                  frameRect,gfx);
               putcel(newx,y,dest,frameRect,cachedgfx);
               cached |= framebit;
            }
         }
         else
         {
            puttransparent(newx,y,dest,frameRect,gfx);
         }
      }
      else
         putcel(newx,y,dest,frameRect,gfx);
      SET_USEDMAP(newx,y);
      SET_USEDMAP(xright,y);
      SET_USEDMAP(newx,ybottom);
      SET_USEDMAP(xright,ybottom);
   }
}


//-------------------------------------------------------------------------
// pohyb bonusu
//-------------------------------------------------------------------------
int Bonus::Move()
{
  int result = 0;

  counter++;
  mul_x += movex;
  newx = mul_x / MUL_MOVE;
  if (newx<PLAYFIELD_MIN_X) {
    newx = PLAYFIELD_MIN_X;
    mul_x = newx * MUL_MOVE;
    movex = -movex;
    result |= SP_COL_L;
  }
  else if (newx>b_maxx) {
    newx = b_maxx;
    mul_x = newx * MUL_MOVE;
    movex = -movex;
    result |= SP_COL_R;
  }
  mul_y += movey;
  newy = mul_y / MUL_MOVE;
  if (newy<PLAYFIELD_EMIN_Y) {
    newy = eb_maxy;
    frameRect.top = frameRect.bottom - ysize;
    mul_y = newy * MUL_MOVE;
    result |= SP_COL_U;
  }
  else if (newy>eb_maxy) {
    newy = PLAYFIELD_EMIN_Y;
    frameRect.bottom = frameRect.top + ysize;
    mul_y = newy * MUL_MOVE;
    result |= SP_COL_D;
  }
  if (action & SPMA_JUMP) {
    Haste(0,gravity);
  }

  return result;
}

//-------------------------------------------------------------------------
// reakce na pozici a okoli bonusu
//-------------------------------------------------------------------------
void Bonus::MapReflex()
{
   // zmeny stavu!
   if (counter >= BONUS_TIME)
   {
      // osetreni dopadu na plosinku
      if ((action & SPMA_JUMP) && IsStable() > 0 && newy > PLAYFIELD_MIN_Y)
      {
         sound_device->Play(2,newx+xsize/2);
         movex = 0;
         newy = (int)((newy-40+ysize)/MAP_CELY) * MAP_CELY+40-ysize;
         mul_y = newy * MUL_MOVE;
         movey = 0;
         action ^= SPMA_JUMP;
         cacheable = true;
      }
      // osetreni sezratelnosti
      type = BONUS;
   }
}
