#include "RGlass.h"
#include "graphdd.h"
#include "application.h"
#include "utils.h"
#include "exceptions.h"
#include <stdio.h>
#include <stdlib.h>


Surface RGlass::bckg_buf = NULL;
int RGlass::maxHeight = 0;
int RGlass::maxWidth = 0;

//-------------------------------------------------------------------------
// vraci X-offset z WORDu skla
//-------------------------------------------------------------------------
inline int GetXOffset(WORD w)
{
	return int(signed char(w));
}

//-------------------------------------------------------------------------
// vraci Y-offset z WORDu skla
//-------------------------------------------------------------------------
inline int GetYOffset(WORD w)
{
	return int(signed char((w >> 8) & 0xFF));
}

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
RGlass::RGlass(int _width, int _height, WORD * _data)
{
	width = _width;
	height = _height;
	data = _data;
	
	if (data == NULL)
		exit(-1);
	
	CalcPitch();
   if (width + 2*xPitch > maxWidth)
      maxWidth = width + 2*xPitch;
   if (height + 2*yPitch > maxHeight)
      maxHeight = height + 2*yPitch;

}

//-------------------------------------------------------------------------
// destruktor
//-------------------------------------------------------------------------
RGlass::~RGlass()
{
   if (bckg_buf != NULL)
   {
      bckg_buf->Release();
      bckg_buf = NULL;
   }
	if (data != NULL)
	{
		delete [] data;
	}
}

//-------------------------------------------------------------------------
// vypocet prostoru, ktery je potreba pro vyrobu bubliny mimo bublinu
//-------------------------------------------------------------------------
void RGlass::CalcPitch(void)
{
	xPitch = 0;
	yPitch = 0;
	
	int x, y;
	for (y=0; y<height; y++)
		for (x=0; x<width; x++)
		{
			WORD w = data[y*width + x];
			if (w == NO_GLASS || w == CLEAR_GLASS)
				continue;
			
			int xoff = GetXOffset(w);
			int yoff = GetYOffset(w);
			int newx = x + xoff;
			int newy = y + yoff;
			
			if (newx < -xPitch)
			{
				xPitch = -newx;
			}
			
			if (newx - width >= xPitch)
			{
				xPitch = newx - width;
			}
			
			if (newy < -yPitch)
			{
				yPitch = -newy;
			}
			
			if (newy - height >= yPitch)
			{
				yPitch = newy - height;
			}
		}
}

//-------------------------------------------------------------------------
// Narhava sklo bublin
//-------------------------------------------------------------------------
BOOL RGlass::LoadGlasses(char *fileName, RGlass **& glasses, int& count)
{
	FILE *file;
	WORD tempw;
   int mw,mh;
   mw = maxWidth;
   mh = maxHeight;
	
	glasses = NULL;
	count = 0;
	
	file = fopen(fileName,"rb");
	if (file == NULL ||
		fseek(file, -2L, SEEK_END) != 0 ||
		fread(&tempw, sizeof(WORD), 1, file) != 1 ||
		(count = tempw, count == 0) ||
		fseek(file, 0L, SEEK_SET) != 0)
	{
		return FALSE;
	}
	
	glasses = new RGlass * [count];
	if(glasses == NULL)
	{
		return FALSE;
	}
	
	int width, height;
	WORD *data = NULL;
	RGlass *glass = NULL;
	int i;
	
	for (i = 0; i < count; i++)
	{
		data = NULL;
		glass = NULL;
		
		if (fread(&tempw, sizeof(WORD), 1, file) != 1)
			goto error;
		width = tempw;
		
		if (fread(&tempw, sizeof(WORD), 1, file) != 1)
			goto error;
		height = tempw;
		
		if (width * height == 0)
			goto error;
		
		data = new WORD [width * height];
		if (data == NULL)
			goto error;
		
		if (fread(data, width * height * sizeof(WORD), 1, file) != 1)
			goto error;
		
		glass = new RGlass(width, height, data);
		if (glass == NULL)
			goto error;
		
		glasses[i] = glass;
	}

   if (mw != maxWidth || mh != maxHeight)
   {
      if (bckg_buf != NULL)
         bckg_buf->Release();

      //sprintf(exc_string,"%dx%d",maxWidth,maxHeight);
      //throw new BubException(exc_string);
      
      DDSURFACEDESC2 ddsd2;
      ZeroMemory(&ddsd2,sizeof(ddsd2));
      ddsd2.dwSize = sizeof(ddsd2);
      ddsd2.dwFlags = DDSD_WIDTH | DDSD_HEIGHT | DDSD_CAPS;
      ddsd2.dwWidth = maxWidth;
      ddsd2.dwHeight = maxHeight;
      ddsd2.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY;
      ddResult = lpDD4->CreateSurface(&ddsd2,&bckg_buf,NULL);
      if (ddResult != DD_OK)
      {
         bckg_buf = NULL;
         throw new BubException("RGlass: CreateSurface() failed!");
      }
   }
	
	fclose(file);
	
	return TRUE;
	
error:
	while (i--)
	{
		delete glasses[i];
	}
	if (glasses != NULL)
		delete [] glasses;
	if (data != NULL)
		delete [] data;
	
	fclose(file);
	
	return FALSE;
}


//-------------------------------------------------------------------------
// vygeneruje bublinu nad background se zadanym sklem a maskou do
// surface result. mask, glass a result maji stejny width a height
// background je na kazde strane vetsi o glass->Get?Pitch
//-------------------------------------------------------------------------
void doBubble (Surface mask, int maskleft, int masktop, 
               Surface background, RECT valid, 
               TintTable tint, RGlass *glass , Surface result)
{
#define GET_DATA(surfacedesc)	((WORD *)(surfacedesc.lpSurface))

   DDSURFACEDESC2 ddsd2;

	int width = glass->GetWidth();
	int height = glass->GetHeight();
	int pitchx = glass->GetXPitch();
	int pitchy = glass->GetYPitch();
	int glassoff=0;
	int resultoff=0;
   bool valid_valid = false;
	WORD maskw;
	WORD glassw;
   WORD *maskd;
   DWORD maskoff;
   DWORD maskdiff;

   if ((valid.left + valid.right + valid.top + valid.bottom) != 0)
   {
      valid_valid = true;
   }

   if (mask != NULL)
   {
      lock(mask,NULL,&ddsd2);
      maskd = GET_DATA(ddsd2);
      long maskp = ddsd2.lPitch/2;
      maskoff = masktop * maskp + maskleft;
      maskdiff =  maskp - width;
   }
   else
   {
      maskw = colorKey;
   }

   lock(result,NULL,&ddsd2);
	WORD *resultd = GET_DATA(ddsd2);
   DWORD resultp = ddsd2.lPitch/2;

   lock(background,NULL,&ddsd2);
   WORD *backgroundd = GET_DATA(ddsd2);
   DWORD bckgloff = ddsd2.lPitch/2;
   int bckgx, bckgy;
   
   for (int j=0; j<height; j++)
   {
      for (int i=0; i<width; i++)
      {
         if (mask != NULL)
         {
            maskw = maskd[maskoff];
            maskoff++;
         }
         if (maskw != colorKey)
         {
            resultd[resultoff] = maskw;
         }
         else
         {				
            glassw = glass->data[glassoff];       
            if (glassw == NO_GLASS)
            {
               resultd[resultoff] = colorKey;
            }
            else if (glassw == CLEAR_GLASS)
            {
               resultd[resultoff] = 
                  tint[backgroundd[(j+pitchy)*bckgloff+i+pitchx]];
            }
            else
            {
               resultd[resultoff] = 0x7FE0;
               bckgy = j+pitchy+GetYOffset(glassw);
               bckgx = i+pitchx+GetXOffset(glassw);
               if (valid_valid && (bckgx < valid.left || bckgx > valid.right ||
                  bckgy < valid.top || bckgy > valid.bottom))
                  resultd[resultoff] = tint[0];
               else
                  resultd[resultoff] = tint[backgroundd[bckgy*bckgloff+bckgx]];
               
            } 
         }
         glassoff++;
         resultoff++;
      }
      maskoff += maskdiff;
      resultoff = (j+1)*resultp;
   } 
   result->Unlock(NULL);
   background->Unlock(NULL);
   if (mask != NULL)
      mask->Unlock(NULL);
}
