/*
 *--------------------------------------------------------------------
 *
 * Menu
 *
 * Usek hry - menu
 *
 * verze   1.1.1 - 09.10.2004
 * autori  Vojvo - Gunny
 *
 *--------------------------------------------------------------------
 * zmeny
   1.1.1	 / drobne zmeny kolem zobrazovani cheatu
   1.1	 / pridana explicitni dealokace gfx pri ukoncovani menu
 * 1.0.1  - snaha o prvni plnou verzi
 * 0.99.5 - zmena roku v copyrightu
 * 0.99.4 - upravy kvuli AVIPlayeru
 * 0.99.1 - demoverze
 * 0.10.3 - nastaveni hlasitosti
 * 0.10.2 / autori zlute 
 *        - screenshot
 *        - stav klaves se cte primo v update
 * 0.9.8 / darken
 * 0.9.7 - heslo
 * 0.9.5 - toppers
 * 0.9.3 - scrollujici menu
 * 0.8.2 - pridany dalsi funkce menu
 * 0.8.1 - pridana funkcionalita jednoducheho menu
 * 0.7.1 - prvni verze
 *--------------------------------------------------------------------
 */

#include <stdio.h>
#include "consts.h"
#include "types.h"
#include "utils.h"
#include "pass.h"
#include "exceptions.h"
#include "graphDD.h"
#include "surfaceManager.h"
#include "wsound.h"
#include "topper.h"
#include "application.h"
#include "game.h"
#include "menu.h"

#define MAINMENU_ITEMS     10
#define SCROLLSTEP         4
#define SCROLLSPEEDSTATIC  12000
#define SCROLLSPEEDDYNAMIC 12
#define SCROLLSPEEDMAX     120000

int menu_x[MAINMENU_ITEMS] = 
{ 320, 620,  20, 620,  20 };
int menu_y[MAINMENU_ITEMS] = 
{ 260,  20, 500, 500,  20 };


void StartToppers()
{
   app->menu->startToppers();
}

void StopToppers()
{
   app->menu->stopToppers();
}

void Volume(BYTE b)
{
   if (sound_device != NULL)
      if (b == 0)
         sound_device->SetVolume(-10000);
      else
         sound_device->SetVolume(((int)b)*200-2000);
}

void CheckValidKeys()
{
   BYTE keys[8], key_c;
   int current, i;
   bool valid = true, validkeys[8];
   
   current = 0;
   for(i=0;i<8;i++)
      validkeys[i] = true;
   
   while (current < 8)
   {
      key_c =  s_key[current/4][current%4];
      for(i=0; i < current; i++)
      {
         if (keys[i] == key_c)
         {
            validkeys[i] = false;
            validkeys[current] = false;
            valid = false;
         }
      }
      keys[current++] = key_c;
   }

   for(i=0;i<8;i++)
      app->menu->keymenu.item[i+1]->invalid = !validkeys[i];

   if (valid)
   {
      ((MenuLineSwitch2 *)app->menu->keymenu.item[9])->setOption1();
      app->menu-> keymenu.item[9]->invalid = false;
      app->menu->keymenu.item[10]->active = 1;
   }
   else
   {
      ((MenuLineSwitch2 *)app->menu->keymenu.item[9])->setOption2();
      app->menu-> keymenu.item[9]->invalid = true;
      app->menu->keymenu.item[10]->active = 0;
      if (app->menu->keymenu.selected == 10)
      {
         app->menu->keymenu.selected = 1;
         app->menu->keymenu.item[10]->isSelected = 0;
      }
   }
}

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Menu::Menu()   
{
   exitmenu = 0;
   exitsys = 0;
}

//-------------------------------------------------------------------------
// inicializuje veci potrebne pro spousteni
//-------------------------------------------------------------------------
void Menu::load()
{
   mainmenu.item[0]=new MenuLine(320,80," Eriador 1996-2004",0,&qfont_r,&qfont_r);
   mainmenu.item[1]=new MenuLineSwitch(320,110,"Start",1,&qfont_y,&qfont_w,&exitmenu);
   mainmenu.item[2]=new MenuLineString(320,140,"Heslo: ",1,&qfont_y,&qfont_w,s_password,11);
   mainmenu.item[3]=new MenuLineString(320,170,"Hr 1: ",1,&qfont_y,&qfont_g,s_name[0],11);
   mainmenu.item[4]=new MenuLineString(320,200,"Hr 2: ",1,&qfont_y,&qfont_b,s_name[1],11);
   mainmenu.item[5]=new MenuLineSubmenuProc(320,230,"Rekordy",1,&qfont_y,&qfont_w,&menu,&toppermenu,StartToppers);
   mainmenu.item[6]=new MenuLineSubmenu(320,260,"Npovda",1,&qfont_y,&qfont_w,&menu,&helpmenu);
   mainmenu.item[7]=new MenuLineSubmenu(320,290,"Autoi",1,&qfont_y,&qfont_w,&menu,&creditmenu);
   mainmenu.item[8]=new MenuLineSubmenu(320,320,"Nastaven",1,&qfont_y,&qfont_w,&menu,&optionmenu); 
   mainmenu.item[9]=new MenuLineSwitch(320,350,"Konec",1,&qfont_y,&qfont_r,&exitsys);
   mainmenu.lines = 10; 
   mainmenu.selected = 1;
   mainmenu.item[mainmenu.selected]->isSelected = 1;

   optionmenu.item[0]=new MenuLine(320,80,"Nastaven",0,&qfont_r,&qfont_r);
   optionmenu.item[1]=new MenuLineSwitch2(320,120,"Bublina chyt draka ",1,&qfont_y,&qfont_w,&s_catchy,"zepedu","vdy");
   optionmenu.item[2]=new MenuLineSwitch2(320,150,"Kapiky ",1,&qfont_y,&qfont_w,&s_drops,"nevznikaj","jsou vude!");
   optionmenu.item[3]=new MenuLineSwitch2(320,180,"Heslo mnit ",1,&qfont_y,&qfont_w,&s_changepass,"se vemi ivoty","vdy");
   optionmenu.item[4]=new MenuLineSwitch2(320,210,"Bubliny ",1,&qfont_y,&qfont_w,&s_solidbubbles,"sklenn","obrysov");
   optionmenu.item[5]=new MenuLineSwitch2(320,240,"Pozad ",1,&qfont_y,&qfont_w,&s_darken,"nemnit","ztmavit");
   optionmenu.item[6]=new MenuLineSubmenu(320,270,"Nastaven zvuku",!sound_device->werror,&qfont_y,&qfont_w,&menu,&soundmenu);
   optionmenu.item[7]=new MenuLineSubmenu(320,300,"Nastaven klves",1,&qfont_y,&qfont_w,&menu,&keymenu);
   optionmenu.item[8]=new MenuLineSubmenu(320,330,"Zptky",1,&qfont_y,&qfont_r,&menu,&mainmenu);
   optionmenu.lines=9;
   optionmenu.selected=8;
   optionmenu.item[optionmenu.selected]->isSelected=1;
   optionmenu.upper=&mainmenu;
   
   soundmenu.item[0]=new MenuLine(320,80,"Nastaven zvuku",0,&qfont_r,&qfont_r);
   soundmenu.item[1]=new MenuLineSwitch2(320,120,"Stereo ",1,&qfont_y,&qfont_w,&(sound_device->reverse_stereo),"normln","obrcen");
   soundmenu.item[2]=new MenuSlider(320,150,"Hlasitost ",1,&qfont_y,&qfont_w,10,&s_vol,Volume);
   soundmenu.item[3]=new MenuLineSubmenu(320,190,"Zptky",1,&qfont_y,&qfont_r,&menu,&optionmenu);
   soundmenu.lines=4;
   soundmenu.selected=3;
   soundmenu.item[soundmenu.selected]->isSelected=1;
   soundmenu.upper=&optionmenu;
   
   creditmenu.item[0]=new MenuLine(320,80,"Autoi",0,&qfont_r,&qfont_r);
   creditmenu.item[1]=new MenuLine(320,120, "Koncept               Eriador",0,&qfont_y,&qfont_y);
   creditmenu.item[2]=new MenuLine(320,145, "Program     Vojvo-Gunny-Rongi",0,&qfont_y,&qfont_y);
   creditmenu.item[3]=new MenuLine(320,170, "Grafika   Gunny-Grumpfh-Nalim",0,&qfont_y,&qfont_y);
   creditmenu.item[4]=new MenuLine(320,195, "Animace           Gunny-Nalim",0,&qfont_y,&qfont_y);
   creditmenu.item[5]=new MenuLine(320,220, "Zvuky       Gunny-Nalim-Lucky",0,&qfont_y,&qfont_y);
   creditmenu.item[6]=new MenuLine(320,245, "Dal programy    Gunny-Vojvo",0,&qfont_y,&qfont_y);
   creditmenu.item[7]=new MenuLine(320,270, "Editory                 Gunny",0,&qfont_y,&qfont_y);
   creditmenu.item[8]=new MenuLine(320,295, "Design level         Eriador",0,&qfont_y,&qfont_y);
   creditmenu.item[9]=new MenuLine(320,320,"Organizace level        Lumi",0,&qfont_y,&qfont_y);
   creditmenu.item[10]=new MenuLine(320,345,"Beta testy    Lumi-Dave-Lenka",0,&qfont_y,&qfont_y);
   creditmenu.item[11]=new MenuLine(320,370,"                  Fajmi-Nalim",0,&qfont_y,&qfont_y);
   creditmenu.item[12]=new MenuLine(320,395," Eriador 1996-2004",0,&qfont_y,&qfont_y);
   creditmenu.item[13]=new MenuLineSubmenu(320,420,"Zptky",1,&qfont_y,&qfont_r,&menu,&mainmenu);
   creditmenu.lines=14;
   creditmenu.selected=13;
   creditmenu.item[creditmenu.selected]->isSelected=1;
   creditmenu.upper=&mainmenu;
   
   helpmenu.item[0]=new MenuLine(320,80,"O co jde",0,&qfont_r,&qfont_r);
   helpmenu.item[1]=new MenuLine(320,110, "Cl hry je naprosto netradin",0,&qfont_y,&qfont_y);
   helpmenu.item[2]=new MenuLine(320,140, "- zlikvidovat potvrky. ",0,&qfont_y,&qfont_y);
   helpmenu.item[3]=new MenuLine(320,170, "Hri ovldaj drky plivajc",0,&qfont_y,&qfont_y);
   helpmenu.item[4]=new MenuLine(320,200, "speciln bubliny, kter jsou",0,&qfont_y,&qfont_y);
   helpmenu.item[5]=new MenuLine(320,230,"schopn potvrky polapit.",0,&qfont_y,&qfont_y);
   helpmenu.item[6]=new MenuLine(320,260,"Pak u sta jen bublinu rozbt a",0,&qfont_y,&qfont_y);
   helpmenu.item[7]=new MenuLine(320,290,"potvrka je zlikvidovan. Pokud ji",0,&qfont_y,&qfont_y);
   helpmenu.item[8]=new MenuLine(320,320,"vak hr vas nerozbije, potvrka",0,&qfont_y,&qfont_y);
   helpmenu.item[9]=new MenuLine(320,350,"se uvoln a utee.",0,&qfont_y,&qfont_y);
   helpmenu.item[10]=new MenuLineSubmenu(320,380,"Dle",1,&qfont_y,&qfont_r,&menu,&help2menu);
   helpmenu.lines=11;
   helpmenu.selected=10;
   helpmenu.item[helpmenu.selected]->isSelected=1;
   helpmenu.upper=&mainmenu;

   help2menu.item[0]=new MenuLine(320,80,"Bonusy",0,&qfont_r,&qfont_r);
   help2menu.item[1]=new MenuLine(320,110, "Pi rozbit obsazen bubliny z n",0,&qfont_y,&qfont_y);
   help2menu.item[2]=new MenuLine(320,140, "vypadne njak ovoce na zub a na",0,&qfont_y,&qfont_y);
   help2menu.item[3]=new MenuLine(320,170, "body a navc se objev speciln",0,&qfont_y,&qfont_y);
   help2menu.item[4]=new MenuLine(320,200, "magick bubliny. Jejich sbrnm",0,&qfont_y,&qfont_y);
   help2menu.item[5]=new MenuLine(320,230,"je mon sestavit kouzelnou",0,&qfont_y,&qfont_y);
   help2menu.item[6]=new MenuLine(320,260,"formuli a nco (ne)uitenho si",0,&qfont_y,&qfont_y);
   help2menu.item[7]=new MenuLine(320,290,"vykouzlit.",0,&qfont_y,&qfont_y);
   help2menu.item[8]=new MenuLine(320,320,"Kouzla si zkuste sami!",0,&qfont_y,&qfont_y);
   help2menu.item[9]=new MenuLineSubmenu(320,360,"Zptky",1,&qfont_y,&qfont_r,&menu,&mainmenu);
   help2menu.lines=10;
   help2menu.selected=9;
   help2menu.item[help2menu.selected]->isSelected=1;
   help2menu.upper=&mainmenu;

   keymenu.item[0]=new MenuLine(320,80,"Nastaven klves",0,&qfont_r,&qfont_r);
   keymenu.item[1]=new MenuLineKey(200,110, "Doleva  ",1,&qfont_y,&qfont_g,(char*)&(s_key[0][0]));
   keymenu.item[2]=new MenuLineKey(200,140, "Doprava ",1,&qfont_y,&qfont_g,(char*)&(s_key[0][1]));
   keymenu.item[3]=new MenuLineKey(200,170, "Skok    ",1,&qfont_y,&qfont_g,(char*)&(s_key[0][2]));
   keymenu.item[4]=new MenuLineKey(200,200, "Plivn ",1,&qfont_y,&qfont_g,(char*)&(s_key[0][3]));
   keymenu.item[5]=new MenuLineKey(440,230,"Doleva  ",1,&qfont_y,&qfont_b,(char*)&(s_key[1][0]));
   keymenu.item[6]=new MenuLineKey(440,260,"Doprava ",1,&qfont_y,&qfont_b,(char*)&(s_key[1][1]));
   keymenu.item[7]=new MenuLineKey(440,290,"Skok    ",1,&qfont_y,&qfont_b,(char*)&(s_key[1][2]));
   keymenu.item[8]=new MenuLineKey(440,320,"Plivn ",1,&qfont_y,&qfont_b,(char*)&(s_key[1][3]));
   keymenu.item[9]=new MenuLineSwitch2(320,360,"Klvesy jsou ",0,&qfont_y,&qfont_w,&s_catchy,"v podku","patn!");
   keymenu.item[10]=new MenuLineSubmenu(320,390,"Zptky",1,&qfont_y,&qfont_r,&menu,&optionmenu);
   keymenu.onUpdate = CheckValidKeys;
   keymenu.lines=11;
   keymenu.selected=10;
   keymenu.item[keymenu.selected]->isSelected=1;
   keymenu.upper=&optionmenu;  

   toppermenu.item[0] = new MenuLine(320,50,"Nejlep draci",0,&qfont_r,&qfont_r);
   toppermenu.item[1] = new MenuLineSubmenuProc(320,410,"Zptky",1,&qfont_y,&qfont_r,&menu,&mainmenu,
      StopToppers);
   toppermenu.item[2] = new MenuLineToppers(180,90,300);
   toppermenu.lines=3;
   toppermenu.selected=1;
   toppermenu.item[toppermenu.selected]->isSelected=1;
   toppermenu.upper=&mainmenu;  
   
   menu = &mainmenu;
}

//-------------------------------------------------------------------------
// inicializuje vypis hi-scores
//-------------------------------------------------------------------------
void Menu::startToppers()
{
   DEBUG_MESSAGE("Menu::startToppers()\n");
   if (toppers == NULL)
      return;
   ((MenuLineToppers *)(toppermenu.item[2]))->start();
   DEBUG_MESSAGE("Menu::startToppers()\n");
}

//-------------------------------------------------------------------------
// deinicializuje vypis hi-scores
//-------------------------------------------------------------------------
void Menu::stopToppers()
{
   DEBUG_MESSAGE("Menu::stopToppers()\n");
   if (toppers == NULL)
      return;
   ((MenuLineToppers *)(toppermenu.item[2]))->stop();
   DEBUG_MESSAGE("Menu::stopToppers()\n");
}

//-------------------------------------------------------------------------
// zahajuje cinnost
//-------------------------------------------------------------------------
void Menu::start()
{
   static bool firsttime = true;
   exitmenu = 0;
   surfaceManager->remove(menugfx, true);
   menugfx = surfaceManager->load("menu2.r16","data\\pictures.dir");
   delete mainmenu.item[2];
   mainmenu.item[2]=new MenuLineString(320,140,"Heslo: ",1,&qfont_y,&qfont_w,s_password,11);
   if (menu == &mainmenu)
   {
      destx = menu_x[0];
      desty = menu_y[0];
   }
   else if (menu == &toppermenu)
   {
      destx = menu_x[1];
      desty = menu_y[1];
   }
   else if (menu == &helpmenu || menu == &help2menu)
   {
      destx = menu_x[2];
      desty = menu_y[2];
   }
   else if (menu == &creditmenu)
   {
      destx = menu_x[3];
      desty = menu_y[3];
   }
   else if (menu == &optionmenu || menu == &keymenu)
   {
      destx = menu_x[4];
      desty = menu_y[4];
   }
   if (destx > 600)
   {
      x.value[1] = (WORD)destx-20;
      movex = -SCROLLSPEEDSTATIC;
   }
   else
   {
      x.value[1] = (WORD)destx+20;
      movex = SCROLLSPEEDSTATIC;
   }
   x.value[0] = 0;
   y.value[1] = (WORD)desty;
   y.value[0] = 0;
   movex = SCROLLSPEEDSTATIC;
   movey = SCROLLSPEEDSTATIC;
   if (firsttime)
   {
      sound_device->Play(30);
      firsttime = false;
   }
   kbd.filter_lastscan = false;
   kbd.lastScanReset();
   kbd.lastASCIIReset();
   this->oldtime = GetTickCount();
}

//-------------------------------------------------------------------------
// pravidelna cinnost pri kazdem pruchodu hlavni smyckou programu
//-------------------------------------------------------------------------
void Menu::update()
{
   // klavesnice
   kbd.getState();

   if (menu->onUpdate != NULL)
      menu->onUpdate();

   if (KEYDOWN(kbd.diKeys,DIK_F9))
   {
      kbd.clearKey(DIK_F9);
	   kbd.lastScanReset();
      filnam[5]=(picnum / 100)+'0';
      filnam[6]=(picnum % 100)/10+'0';
      filnam[7]=(picnum % 10)+'0';
      screenshot(backbuffer,filnam);
      picnum++;
   }
   if (KEYDOWN(kbd.diKeys,DIK_ESCAPE))
   {
      if (menu != &keymenu || keymenu.item[10]->active != 0)
      {
         if (menu == &mainmenu)
            exitsys = 1;
         else
            menu=menu->upper;
      }
      kbd.clearKey(DIK_ESCAPE);
   }
   if (s_test && menu==&mainmenu && KEYDOWN(kbd.diKeys,DIK_F11))
   {
      kbd.clearKey(DIK_F11);
      s_testsymbol = !s_testsymbol;
      strcpy(mainmenu.item[0]->text,"Test: xx");
      if (s_testsymbol)
			mainmenu.item[0]->text[6] = 'S';
      if (s_testholy)
			mainmenu.item[0]->text[7] = 'N';
   }
   if (s_test && menu==&mainmenu && KEYDOWN(kbd.diKeys,DIK_F12))
   {
      kbd.clearKey(DIK_F12);
      s_testholy = !s_testholy;
      strcpy(mainmenu.item[0]->text,"Test: xx");
      if (s_testsymbol)
			mainmenu.item[0]->text[6] = 'S';
      if (s_testholy)
			mainmenu.item[0]->text[7] = 'N';
   }         

   if (exitsys)
   {
      app->stateChange(REASON_EXIT);
      return;
   }

   if (exitmenu)
   {
      app->stateChange();
      return;
   }

   DWORD deltatime;
   BYTE scrollsteps;
   int diffx,diffy;

   deltatime = GetTickCount() - this->oldtime;
   if (deltatime > 1000)
   {
      this->oldtime = GetTickCount();
      scrollsteps = 0;
   }
   scrollsteps = (BYTE)(deltatime/SCROLLSTEP);
   if (scrollsteps > 0)
   {
      this->oldtime = GetTickCount();
      for(BYTE i=0; i<scrollsteps; i++)
      {
         diffx = (int)(destx - x.value[1]);
         if (diffx > 100 || diffx < -100)
            maxspeedx = SCROLLSPEEDMAX;
         else
            maxspeedx = SCROLLSPEEDSTATIC + diffx*diffx*SCROLLSPEEDDYNAMIC;
         movex += diffx * abs(diffx);
         if (movex > 0) 
         {
            if (movex > maxspeedx) 
               movex = maxspeedx;
         }
         else
         {
            if (movex<-maxspeedx) 
               movex=-maxspeedx;
         }
         x.mulvalue += movex;

         diffy = (int)(desty - y.value[1]);         
         if (diffy > 100 || diffy < -100)
            maxspeedy = SCROLLSPEEDMAX;
         else
         maxspeedy = SCROLLSPEEDSTATIC + diffy*diffy*SCROLLSPEEDDYNAMIC;
         movey += diffy * abs(diffy);
         if (movey > 0)
         {
            if (movey > maxspeedy) 
               movey = maxspeedy;
         }
         else
         {
            if (movey<-maxspeedy) 
               movey=-maxspeedy;
         }
         y.mulvalue += movey;
      }
   }
         
   //--------------------------------------------------------- paint
   if (menu==&toppermenu)
      MenuLineToppers::ComputeSteps();
   RECT r;
   r.left = x.value[1];
   r.right = x.value[1]+640;
   r.top = y.value[1];
   r.bottom = y.value[1]+480;
   putcel(0,0,backbuffer,r,menugfx,DDBLTFAST_NOCOLORKEY);
   menu->Paint(backbuffer);
   flip(0);

   //---------------------------------------------------------- akce
   menu->KeyReflex();
   if (menu == &mainmenu)
   {
      destx = menu_x[0];
      desty = menu_y[0];
   }
   else if (menu == &toppermenu)
   {
      destx = menu_x[1];
      desty = menu_y[1];
   }
   else if (menu == &helpmenu || menu == &help2menu)
   {
      destx = menu_x[2];
      desty = menu_y[2];
   }
   else if (menu == &creditmenu)
   {
      destx = menu_x[3];
      desty = menu_y[3];
   }
   else if (menu == &optionmenu || menu == &keymenu)
   {
      destx = menu_x[4];
      desty = menu_y[4];
   }
}

//-------------------------------------------------------------------------
// obnoveni po surface lost
//-------------------------------------------------------------------------
void Menu::restore()
{
   if (menu == &toppermenu)
   {
      ((MenuLineToppers *)(toppermenu.item[2]))->start();
   }
}

//-------------------------------------------------------------------------
// cinnost pri zastaveni zobrazovani objektu
//-------------------------------------------------------------------------
void Menu::stop()
{
	// V. 2004
   surfaceManager->remove(menugfx, true);
	//
   kbd.filter_lastscan = true;
}

//-------------------------------------------------------------------------
// cinnost pri definitivnim ukonceni pouzivani
//-------------------------------------------------------------------------
void Menu::unload()
{
   
}
