//****************************************************************************
//                     Veci, ktere se nejak tykaji menu                     
//****************************************************************************
// Written by : Gunny & Vojvo                                                
// Language   : VisuC + DX6.1                                                 
// Version    : 1.1.1
// Last update: 10.10.2004                                                      
//****************************************************************************
/*
// changes:
   1.1.1 / zrusena zbytecna promenna panpos
// 0.99.4 - uprava kvuli AVIPlayeru
// 0.9.5 / opraven bug v menulinestring (doufejme)
// 0.9.4 - pridana trida MenuLineScrolling
//       - upravena trida MenuLineSubmenuProc
// 0.8.4 - pridany zvuky
// 0.8.3 - MenuLineSwitch2 pracuje s boolem
*/

#include <stdio.h> 
#include <string.h>
#include "consts.h"
#include "exceptions.h"
#include "utils.h"
#include "graphDD.h"
#include "surfaceManager.h"
#include "font.h"
#include "keyboard.h"
#include "wsound.h"
#include "topper.h"
#include "menuobject.h"


#define MENU_CURSORNORMAL    '|'
#define MENU_CURSORBLINK     ' '
#define MENU_CURSORNONE      ' '

#define TOPPER_SOUNDINTERVAL  80

BYTE menu_key;
bool ignore_menu = false;

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
MenuObject::~MenuObject(void)
{
   for (int i=0; i<lines; i++) 
   {
      delete(item[i]);
      item[i] = NULL;
   }
   lines = 0;
}

//-------------------------------------------------------------------------
// reakce na klavesy
//-------------------------------------------------------------------------
void MenuObject::KeyReflex()
{ 
   menu_key = kbd.LastScan();
   if (menu_key == 0)
      return;
   
   if (ignore_menu)
   {
      item[selected]->KeyReflex();
      return;
   }
   
   if (menu_key == DIK_UP)
   {
      kbd.lastASCIIReset();
      item[selected]->isSelected=0;
      if (item[selected]->cursor)
         item[selected]->changed = 1;
      sound_device->Play(13);
      do
      {
         selected--;
         if (selected<0) selected=lines-1;
      } 
      while (!(item[selected]->active));
      item[selected]->isSelected=1;
   }
   if (menu_key == DIK_DOWN)
   {
      kbd.lastASCIIReset();
      item[selected]->isSelected=0;
      if (item[selected]->cursor)
         item[selected]->changed = 1;
      sound_device->Play(13);
      do
      {   
         selected++;
         if (selected>=lines) selected=0;
      } while (!(item[selected]->active));
      item[selected]->isSelected=1;
   }
   //
   if (menu_key == DIK_HOME)
   {
      kbd.lastASCIIReset();
      item[selected]->isSelected=0;
      if (item[selected]->cursor)
         item[selected]->changed = 1;
      sound_device->Play(13);
      selected=0;
      while (!(item[selected]->active))
      {
         selected++;
         if (selected>=lines) selected=0;
      }
      item[selected]->isSelected=1;
   }
   if (menu_key == DIK_END)
   {
      kbd.lastASCIIReset();
      item[selected]->isSelected=0;
      if (item[selected]->cursor)
         item[selected]->changed = 1;
      sound_device->Play(13);
      selected=lines-1;
      while (!(item[selected]->active))
      {
         selected--;
         if (selected<0) selected=lines-1;
      }
      item[selected]->isSelected=1;
   }
   item[selected]->KeyReflex();
}

//-------------------------------------------------------------------------
// vykresleni menu
//-------------------------------------------------------------------------
void MenuObject::Paint(Surface dest)
{
   for (int i=0; i<lines; i++) 
      item[i]->Paint(dest);
}

//-------------------------------------------------------------------------
// smazani menu - zmenenych polozek
//-------------------------------------------------------------------------
void MenuObject::Clear(Surface dest, Surface bckg)
{
   for (int i=0; i<lines; i++)
      if (item[i]->changed) 
         item[i]->Clear(dest,bckg);
}

//------------------------------------------------------------------------
// smazani menu - vsech polozek
//------------------------------------------------------------------------
void MenuObject::ClearAll(Surface dest, Surface bckg)
{
   for (int i=0; i<lines; i++)
      item[i]->Clear(dest,bckg);
}

//========================================================================
//-------------------------------------------------------------------------
// MenuLine - radek menu
//-------------------------------------------------------------------------
//========================================================================

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
MenuLine::MenuLine(int ix, int iy, char *txt, int a, Font *f1, Font *f2)
{
   x=ix;
   y=iy;
   strcpy(text,txt);
   active=a;
   fnt1=f1;
   fnt2=f2;
   isSelected=0;
   len=strlen(text);
   r.left = 0;
   r.right = 640;
   r.top = iy;
   r.bottom = iy + max(fnt1->ysize,fnt2->ysize);
   changed=0;
   cursor = 0;
   invalid = false;
}

//-------------------------------------------------------------------------
// reakce na stisk klavesy
//-------------------------------------------------------------------------
void MenuLine::KeyReflex()
{
}

//-------------------------------------------------------------------------
// vykresleni radky menu
//-------------------------------------------------------------------------
void MenuLine::Paint(Surface dest)
{
   Font *f;
   if (isSelected) 
      f = fnt2;
   else if (invalid)
      f = &qfont_r;
   else
      f = fnt1;
   putStringCentered(x,y,dest,text,*f);
}

//-------------------------------------------------------------------------
// smazani radky 
//-------------------------------------------------------------------------
void MenuLine::Clear(Surface dest, Surface bckg)
{
   sprintf(exc_string,"%p,%p",dest,bckg);
   throw new BubException(exc_string);
   putcel(0,y,dest,r,bckg,DDBLTFAST_NOCOLORKEY);
   changed=0;
}

//=========================================================================
//-------------------------------------------------------------------------
// Menu Line Toppers - radka menu - rekordy
//-------------------------------------------------------------------------
//=========================================================================
DWORD MenuLineToppers::oldtime = 0;
DWORD MenuLineToppers::scrollsteps = 0;
bool MenuLineToppers::stopped = false;

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
MenuLineToppers::MenuLineToppers(int ix, int iy,int wheight)
:MenuLine(ix,iy,"toppers",0,&qfont_w,&qfont_w)
{
   surf = NULL;
   rWindow.left = 0;
   rWindow.right = 300;
   rWindow.top = 799;
   rWindow.bottom = 800;   
   height = wheight;
}

//-------------------------------------------------------------------------
// inicializuje casovani scrollovani
//-------------------------------------------------------------------------
void MenuLineToppers::InitSteps()
{
   oldtime = GetTickCount();
   scrollsteps = 0;
}

//-------------------------------------------------------------------------
// casovani scrollovani
//-------------------------------------------------------------------------
void MenuLineToppers::ComputeSteps()
{
   DWORD time;
   time = GetTickCount();
   scrollsteps = (time-oldtime)/TOPPERS_SCROLLSPEED;
   if (scrollsteps > 0 && !stopped)
      oldtime = time;
   if (stopped && scrollsteps >= 100)
   {
      stopped = false;
      scrollsteps = 0;
      oldtime = time;
   } 
}

//-------------------------------------------------------------------------
// pripravuje toppers pro vykreslovani
//-------------------------------------------------------------------------
void MenuLineToppers::start()
{
   surfaceManager->remove("*toppers");
   surf = NULL;
   surf = surfaceManager->create(300,800,"*toppers");
   setColorKey(surf,colorKey);
   clear(surf,colorKey,NULL);
   for (int i=0; i < TOP_MAXTOP; i++)
   {
      if (toppers->data[i] != NULL)
      {
         sprintf(pom_string,"%2d %-11s  %8ld",i+1,toppers->data[i]->name,
            toppers->data[i]->score);
         putString(0,i*20,surf,pom_string,*(toppers->data[i]->fnt),false);
      }
   }
   rWindow.left = 0;
   rWindow.right = 300;
   rWindow.top = 799;
   rWindow.bottom = 800;
   lastSoundl = 800/TOPPER_SOUNDINTERVAL;
   lastSoundr = 825/TOPPER_SOUNDINTERVAL;
   stopped = false;
   lastStop = 0;
   InitSteps();
}

//-------------------------------------------------------------------------
// uklizi po ukonceni vykreslovani toppers
//-------------------------------------------------------------------------
void MenuLineToppers::stop()
{
   if (surf != NULL)
      surfaceManager->remove(surf);
}

//-------------------------------------------------------------------------
// vykresleni toppers
//-------------------------------------------------------------------------        
void MenuLineToppers::Paint(Surface dest)
{
   int current;
   if (!stopped)
   {
      if (scrollsteps > 0)
      {
         rWindow.top -= (int)scrollsteps;
         if (rWindow.top < 0)
         {
            rWindow.top = 0;
            if (lastSoundl >= 0)
            {
               sound_device->Play(24);
               lastSoundl = -1;
            }
         }
         if (rWindow.top/TOPPER_SOUNDINTERVAL < lastSoundl)
         {
            sound_device->Play(25,240);
            lastSoundl = rWindow.top/TOPPER_SOUNDINTERVAL;
         }
         if ((rWindow.top+25)/TOPPER_SOUNDINTERVAL < lastSoundr)
         {
            sound_device->Play(25,400);
            lastSoundr = (rWindow.top+25)/TOPPER_SOUNDINTERVAL;
         }
         rWindow.bottom = rWindow.top + height;
         if (rWindow.bottom > 800)
            rWindow.bottom = 800;
         current = (rWindow.top / 20)+1;
         if (current < TOP_MAXTOP && toppers->data[current]->newone 
            && current != lastStop)
         {
            sound_device->Play(24);
            stopped = true;
            lastStop = current;
         }
      }
   }
   putcel(x,y,dest,rWindow,surf);
}

//=========================================================================
//-------------------------------------------------------------------------
// Menu Line String - radka menu - text
//-------------------------------------------------------------------------
//=========================================================================
MenuLineString::MenuLineString(int ix, int iy, char *txt, int a, Font *f1, 
                               Font *f2, char *v, int ml)
                               : MenuLine(ix,iy,txt,a,f1,f2)
{
   var=v;
   strcpy(label,txt);
   strcpy(text,label);   
   strcat(text,v);
   len=strlen(text);
   r.left = 0;
   r.right = 640;
   r.top = iy;
   r.bottom = iy + max(fnt1->ysize,fnt2->ysize);
   cursor = 1;
   pulse = 0;
   maxlen = ml;
}

void MenuLineString::Paint(Surface dest)
{
   if (isSelected)
   {
      pulse++;
      if (pulse>12)
      {
         pulse = 0;
         text[len] = MENU_CURSORNORMAL;
      }
      else if (pulse>8) 
      {
         text[len] = MENU_CURSORBLINK;
      }
   }
   else
      text[len] = MENU_CURSORNONE;
   text[len+1] = 0;
   MenuLine::Paint(dest);
}

void MenuLineString::KeyReflex()
{
   BYTE key;
   key = kbd.LastASCII();
   if (key != 0)
   {
      int varlen=strlen(var);
      if (key >= 32)
      {
         if (varlen<maxlen)
         {
            var[varlen] = key;
            var[varlen+1]=0;
            sound_device->Play(17);
         }
         else 
         {
            sound_device->Play(18);
         }
      }
      if (key == 8 && varlen>0)
      {
         var[varlen-1]=0;
         sound_device->Play(15);
      }
      strcpy(text,label);
      strcat(text,var);
      len = strlen(text);
      text[len] = MENU_CURSORNORMAL;
      text[len+1] = '\x0';
      changed=1;
   }
}

//---------------------------------------------------------------------------

MenuLineSwitch::MenuLineSwitch(int ix, int iy, char *txt, int a, Font *f1, 
                               Font *f2, char *v)
                               : MenuLine(ix,iy,txt,a,f1,f2)
{
   var=v;
}

void MenuLineSwitch::KeyReflex()
{
   BYTE key;
   key = kbd.LastASCII();
   if (key == 13 || key == 32)
   {
      *var=!(*var);
      sound_device->Play(14);
   }
}

//---------------------------------------------------------------------------

MenuLineSwitch2::MenuLineSwitch2(int ix, int iy, char *txt, int a, Font *f1, 
                                 Font *f2, bool *v, char *o1, char *o2)
                                 : MenuLine(ix,iy,txt,a,f1,f2)
{
   varb = v;
   strcpy(label,txt);
   strcpy(option1,o1);
   strcpy(option2,o2);
   if (*varb) 
      strcat(text,option2);
   else 
      strcat(text,option1);
   len=strlen(text);
}

void MenuLineSwitch2::KeyReflex()
{
   BYTE key = kbd.LastASCII();
   if (key == 13)
   {
      *varb=!(*varb);
      sound_device->Play(14);
      strcpy(text,label);
      if (*varb) 
         strcat(text,option2);
      else 
         strcat(text,option1);
      len = strlen(text);
      changed=1;
   }
}

void MenuLineSwitch2::setOption1()
{
   strcpy(text,label);
   strcat(text,option1);
   len = strlen(text);
   changed=1;
}

void MenuLineSwitch2::setOption2()
{
   strcpy(text,label);
   strcat(text,option2);
   len = strlen(text);
   changed=1;
}

//---------------------------------------------------------------------------

MenuLineSwitchProc::MenuLineSwitchProc(int ix, int iy, char *txt, int a, 
                                       Font *f1, Font *f2, char *v,
                                       char *o1, char *o2, void (*p)(char))
                                       : MenuLineSwitch(ix,iy,txt,a,f1,f2,v)
{
   proc = p;
   strcpy(label,txt);
   strcpy(option1,o1);
   strcpy(option2,o2);
   if (*var) 
      strcat(text,option2);
   else 
      strcat(text,option1);
   len=strlen(text);
}

void MenuLineSwitchProc::KeyReflex()
{
   BYTE key = kbd.LastASCII();
   if (key == 13)
   {
      *var=!(*var);
      sound_device->Play(14);
      strcpy(text,label);
      if (*var) 
         strcat(text,option2);
      else 
         strcat(text,option1);
      len = strlen(text);
      changed=1;
   }
   if (changed)
      proc(*var);
}

//---------------------------------------------------------------------------

MenuLineSubmenu::MenuLineSubmenu(int ix, int iy, char *txt, int a, Font *f1,
                                 Font *f2, MenuObject **v, MenuObject *nv)
                                 : MenuLine(ix,iy,txt,a,f1,f2)
{
   var=v;
   newvar=nv;
}

void MenuLineSubmenu::KeyReflex()
{
   if (menu_key == DIK_RETURN)
   {
      kbd.lastASCIIReset();
      kbd.lastScanReset();
      sound_device->Play(14);
      *var=newvar;
   }
}

//---------------------------------------------------------------------------

MenuLineSubmenuProc::MenuLineSubmenuProc(int ix, int iy, char *txt,
                                         int a, Font *f1, Font *f2, MenuObject **v, MenuObject *nv,
                                         void (*p)(void))
                                         : MenuLine(ix,iy,txt,a,f1,f2)
{
   var=v;
   newvar=nv;
   proc=p;
}

void MenuLineSubmenuProc :: KeyReflex()
{
   if (menu_key == DIK_RETURN)
   {
      kbd.lastASCIIReset();
      kbd.lastScanReset();
      sound_device->Play(14);
      *var=newvar;
      proc();
   }
}

//---------------------------------------------------------------------------

MenuSlider::MenuSlider(int ix, int iy, char *txt, int a, Font *f1, Font *f2,
                       int m, BYTE *v, void (*p)(BYTE))
                       : MenuLine(ix,iy,txt,a,f1,f2)
{
   strcpy(label,txt);
   int sl=strlen(label);
   max=m;
   var=v;
   for (int i=0;i<*var;i++) text[sl+i]='=';
   text[sl+i]=0;
   len=strlen(text);
   proc=p;
}

void MenuSlider :: KeyReflex()
{
   if (menu_key == DIK_LEFT)
   {
      if (*var>0)
      {
         sound_device->Play(16);
         (*var)--;
         proc(*var);
         text[--len]=0;
         changed=1;
      }
      else 
      {
         sound_device->Play(14);
      }
   }
   if (menu_key == DIK_RIGHT)
   {
      if (*var<max)
      {
         sound_device->Play(16);
         (*var)++;
         proc(*var);
         text[len]='=';
         text[++len]=0;
         changed=1;
      }
      else 
      {
         sound_device->Play(14);
      }
   }
}

//---------------------------------------------------------------------------
MenuLineKey::MenuLineKey(int ix, int iy, char *txt, int a, Font *f1, Font *f2,
                         char *v)
                         : MenuLineSwitch(ix,iy,txt,a,f1,f2,v)
{
   strcpy(label,txt);
   MakeLabel();
   expect=0;
   len=strlen(text);
}

void MenuLineKey::MakeLabel()
{
   char *wrk;
   strcpy(text,label);
   wrk=text+strlen(text);
   *wrk='<'; wrk++;
   *wrk=AsKey[*((BYTE*)var)]; wrk++;
   *wrk='>'; wrk++;
   *wrk=0;
}

void MenuLineKey :: KeyReflex()
{
   if (expect && menu_key!=0 && (menu_key<DIK_F1 || menu_key>DIK_F10)
      && menu_key!=DIK_TAB && menu_key!=DIK_F11 && menu_key!=DIK_F12
      && menu_key!=DIK_ESCAPE)
   {
      int kk;
      expect=0;
      // nacteni vytouzene klavesy
      sound_device->Play(14);
      kk= menu_key;
      if (!kk)
      {
         sound_device->Play(8);
      }
      kbd.lastASCIIReset();
      kbd.lastScanReset();
      *var=kk;
      //update stringu
      MakeLabel();
      changed=1;
      len=strlen(text);  
      ignore_menu = false;
   }
   else
   {
      if (menu_key == DIK_RETURN)
      {
         //SoundDevice->Play(14);
         expect = 1;
         strcpy(text,label);
         strcat(text,"???");
         len = strlen(text);
         changed = 1;
         kbd.lastASCIIReset();
         kbd.lastScanReset();
         ignore_menu = true;
      }
   }
}