#ifndef MENUOBJECT_H_BUBWINY
#define MENUOBJECT_H_BUBWINY

#define MAXMENU	42

#include "types.h"
#include "gamestage.h"

class MenuLine {
public:
   int x,y;          //pocatecni pozice (nebo tak neco)
   int active;		   //jde na neho najet kurzorem?
   int changed;		//nastavi se pri zmene (aby se smazal stary text)
   char text[35];		//zakladni text pro menu
   Font *fnt1;		   //barva v klidu
   Font *fnt2;		   //barva aktivniho radku
   char isSelected;	//bool, jestli je na nem kurzor
   bool invalid;     //je radek neplatny|nespravny?
   int  len;		   //uchovava puldelku v pixlech z min. vykresleni
   int  cursor;      //priznak kurzoru - pri opusteni je nutno smazat
   RECT r;           //pracovni oblast pro mazani menuline
		
	MenuLine(int ix, int iy, char *txt, int a, Font *f1, Font *f2);
	virtual void KeyReflex();           //reakce na klavesy (hlavne enter =)
	virtual void Paint(Surface dest);   //vykresli radek
	void Clear(Surface dest, Surface bckg);//smaze
	virtual ~MenuLine() {};
};

class MenuObject
{
public:
   MenuLine *item[MAXMENU];   //pointry na jednotlive radky
   int lines;		            //pocet pouzitych radku
   int selected;		         //vybrany radek
   MenuObject *upper;	      //pod kterym menu jsme zanoreni (kam se vracet)
   void (*onUpdate)(void);    //akce, ktera se provadi pri kazdem obnoveni menu
	  
   void KeyReflex();	         //reakce na klavesy (kurzor, zbytek dela radek)
   void Paint(Surface dest);	//vykresli menu
   void Clear(Surface dest, Surface bckg);    //smaze (co se bude menit)
   void ClearAll(Surface dest, Surface bckg); //smaze vsechno
   MenuObject() 
   {
      lines=0; 
      selected=1; 
      upper=this; 
      onUpdate=NULL;
   }
   ~MenuObject();
};

//-dal jsou vydedene typy----------------------------------------------------


class MenuLineToppers : public MenuLine {
public:
   static DWORD oldtime, scrollsteps;
   static bool stopped;
   Surface surf;
   RECT rWindow;
   int height;
   int lastStop;
   int lastSoundl;
   int lastSoundr;
   
   MenuLineToppers(int ix, int iy, int wheight);

   static void InitSteps();
   static void ComputeSteps();
   void start();
   void stop();
   virtual void Paint(Surface dest);
};


class MenuLineString : public MenuLine {
public:
   char label[32];      //pevna cast stringu
   char *var;           //pointer na string
   int  pulse;          //blikani kurzoru
   int  maxlen;         //maximalni nacitana delka
	
   MenuLineString(int ix, int iy, char *txt, int a, Font *f1, Font *f2, 
      char *v, int ml);
   virtual void Paint(Surface dest);    //vykresli radek
   virtual void KeyReflex();            //reakce na ASCII
};

class MenuLineSwitch : public MenuLine {
public:
   char *var;		//pointer na switchovanou promennou
	
	MenuLineSwitch(int ix, int iy, char *txt, int a, Font *f1, Font *f2, char *v);
   virtual void KeyReflex();       //reakce na ASCII
};

class MenuLineSwitch2 : public MenuLine {
public:
   char label[32];		//pevna cast stringu
   char option1[16];		//popis 1
   char option2[16];		//popis 2
   bool *varb;          ////pointer na switchovanou promennou
	
	MenuLineSwitch2(int ix, int iy, char *txt, int a, Font *f1, Font *f2, 
      bool *v, char *o1, char *o2);
   virtual void KeyReflex();       //reakce na ASCII
   virtual void setOption1();
   virtual void setOption2();
};

class MenuLineSwitchProc : public MenuLineSwitch {
public:
   char label[32];		//pevna cast stringu
   char option1[16];		//popis 1
   char option2[16];		//popis 2
   void (*proc)(char);         //pointer na pointer funkce vyvolane pri zmene
	
   MenuLineSwitchProc(int ix, int iy, char *txt, int a, Font *f1, Font *f2, 
      char *v, char *o1, char *o2, void (*p)(char));
   virtual void KeyReflex();   //reakce na ASCII
};

class MenuLineSubmenu : public MenuLine {
public:
   MenuObject **var;    //pointer na pointer menu
   MenuObject *newvar;  //menu, ktere nastavi misto neho
	
   MenuLineSubmenu(int ix, int iy, char *txt, int a, Font *f1, Font *f2, 
      MenuObject **v, MenuObject *nv);
   virtual void KeyReflex();       //reakce na ASCII
};

class MenuLineSubmenuProc : public MenuLine {
public:
   MenuObject **var;          //pointer na pointer menu
	MenuObject *newvar;        //menu, ktere nastavi misto neho
	void (*proc)(void);  //pointer na pointer menu
	
   MenuLineSubmenuProc(int ix, int iy, char *txt, int a, Font *f1, Font *f2, 
      MenuObject **v, MenuObject *nv, void (*p)(void));
   virtual void KeyReflex();  //reakce na ASCII
};

class MenuSlider : public MenuLine {
public:
   char label[32];         //pevna cast stringu
   int max;                //maximalni hodnota slideru
   void (*proc)(BYTE);     //pointer na pointer menu
   BYTE *var;              //pointer na promennou
	
   MenuSlider(int ix, int iy, char *txt, int a, Font *f1, Font *f2, int m, 
      BYTE *v, void (*p)(BYTE));
   virtual void KeyReflex();  //reakce na ASCII
};

class MenuLineKey : public MenuLineSwitch {
public:
   char label[16];            //pevna cast stringu (nazev klavesy)
   char expect;
	
   MenuLineKey(int ix, int iy, char *txt, int a, Font *f1, Font *f2, char *v);
	void MakeLabel();          //vyrobi novou popisku
	virtual void KeyReflex();  //reakce na ASCII
};

#endif